@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: closure.ml,v 1.44 2004/05/26 11:10:27 garrigue Exp $ *)
+(* $Id: closure.ml,v 1.48 2005/10/24 09:05:27 xleroy Exp $ *)
 
 (* Introduction of closures, uncurrying, recognition of direct calls *)
 
@@ -240,7 +240,8 @@ let simplif_prim p (args, approxs as args_approxs) =
    clashes with locally-generated identifiers.
    The variables must not be assigned in the term.
    This is used to substitute "trivial" arguments for parameters
-   during inline expansion. *)
+   during inline expansion, and also for the translation of let rec
+   over functions. *)
 
 let approx_ulam = function
     Uconst(Const_base(Const_int n)) -> Value_integer n
@@ -258,15 +259,29 @@ let rec substitute sb ulam =
   | Ugeneric_apply(fn, args) ->
       Ugeneric_apply(substitute sb fn, List.map (substitute sb) args)
   | Uclosure(defs, env) ->
-      (* never present in an inlined function body; painful to get right *)
-      assert false
+      (* Question: should we rename function labels as well?  Otherwise,
+         there is a risk that function labels are not globally unique.
+         This should not happen in the current system because:
+         - Inlined function bodies contain no Uclosure nodes
+           (cf. function [lambda_smaller])
+         - When we substitute offsets for idents bound by let rec
+           in [close], case [Lletrec], we discard the original
+           let rec body and use only the substituted term. *)
+      Uclosure(defs, List.map (substitute sb) env)
   | Uoffset(u, ofs) -> Uoffset(substitute sb u, ofs)
   | Ulet(id, u1, u2) ->
       let id' = Ident.rename id in
       Ulet(id', substitute sb u1, substitute (Tbl.add id (Uvar id') sb) u2)
   | Uletrec(bindings, body) ->
-      (* never present in an inlined function body; painful to get right *)
-      assert false
+      let bindings1 =
+        List.map (fun (id, rhs) -> (id, Ident.rename id, rhs)) bindings in
+      let sb' =
+        List.fold_right 
+          (fun (id, id', _) s -> Tbl.add id (Uvar id') s)
+          bindings1 sb in
+      Uletrec(
+        List.map (fun (id, id', rhs) -> (id', substitute sb' rhs)) bindings1,
+        substitute sb' body)
   | Uprim(p, args) ->
       let sargs = List.map (substitute sb) args in
       let (res, _) = simplif_prim p (sargs, List.map approx_ulam sargs) in
@@ -324,21 +339,27 @@ let no_effects = function
   | Uconst(Const_base(Const_string _)) -> true
   | u -> is_simple_argument u
 
-let rec bind_params subst params args body =
+let rec bind_params_rec subst params args body =
   match (params, args) with
     ([], []) -> substitute subst body
   | (p1 :: pl, a1 :: al) ->
       if is_simple_argument a1 then
-        bind_params (Tbl.add p1 a1 subst) pl al body
+        bind_params_rec (Tbl.add p1 a1 subst) pl al body
       else begin
         let p1' = Ident.rename p1 in
-        let body' = bind_params (Tbl.add p1 (Uvar p1') subst) pl al body in
+        let body' =
+          bind_params_rec (Tbl.add p1 (Uvar p1') subst) pl al body in
         if occurs_var p1 body then Ulet(p1', a1, body')
         else if no_effects a1 then body'
         else Usequence(a1, body')
       end
   | (_, _) -> assert false
 
+let bind_params params args body =
+  (* Reverse parameters and arguments to preserve right-to-left
+     evaluation order (PR#2910). *)
+  bind_params_rec Tbl.empty (List.rev params) (List.rev args) body
+
 (* Check if a lambda term is ``pure'',
    that is without side-effects *and* not containing function definitions *)
 
@@ -359,7 +380,7 @@ let direct_apply fundesc funct ufunct uargs =
   let app =
     match fundesc.fun_inline with
       None -> Udirect_apply(fundesc.fun_label, app_args)
-    | Some(params, body) -> bind_params Tbl.empty params app_args body in
+    | Some(params, body) -> bind_params params app_args body in
   (* If ufunct can contain side-effects or function definitions,
      we must make sure that it is evaluated exactly once.
      If the function is not closed, we evaluate ufunct as part of the
@@ -487,11 +508,12 @@ let rec close fenv cenv = function
             (fun (id, pos, approx) fenv -> Tbl.add id approx fenv)
             infos fenv in
         let (ubody, approx) = close fenv_body cenv body in
-        (Ulet(clos_ident, clos,
-              List.fold_right
-                (fun (id, pos, approx) body ->
-                    Ulet(id, Uoffset(Uvar clos_ident, pos), body))
-                infos ubody),
+        let sb =
+          List.fold_right
+            (fun (id, pos, approx) sb ->
+              Tbl.add id (Uoffset(Uvar clos_ident, pos)) sb)
+            infos Tbl.empty in
+        (Ulet(clos_ident, clos, substitute sb ubody),
          approx)
       end else begin
         (* General case: recursive definition of values *)
@@ -529,7 +551,7 @@ let rec close fenv cenv = function
        Value_unknown)
   | Lprim(p, args) ->
       simplif_prim p (close_list_approx fenv cenv args)
-  | Lswitch(arg, sw) as l ->
+  | Lswitch(arg, sw) ->
 (* NB: failaction might get copied, thus it should be some Lstaticraise *)
       let (uarg, _) = close fenv cenv arg in
       let const_index, const_actions =
@@ -615,7 +637,7 @@ and close_functions fenv cenv fun_defs =
   let uncurried_defs =
     List.map
       (function
-          (id, (Lfunction(kind, params, body) as def)) ->
+          (id, Lfunction(kind, params, body)) ->
             let label = Compilenv.make_symbol (Some (Ident.unique_name id)) in
             let arity = List.length params in
             let fundesc =
