@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: signals.c,v 1.46 2004/01/08 22:28:48 doligez Exp $ */
+/* $Id: signals.c,v 1.51 2005/10/12 12:33:47 xleroy Exp $ */
 
 #include <signal.h>
 #include "alloc.h"
@@ -23,43 +23,78 @@
 #include "mlvalues.h"
 #include "roots.h"
 #include "signals.h"
+#include "signals_machdep.h"
 #include "sys.h"
 
+#ifndef NSIG
+#define NSIG 64
+#endif
+
 #ifdef _WIN32
 typedef void (*sighandler)(int sig);
 extern sighandler caml_win32_signal(int sig, sighandler action);
 #define signal(sig,act) caml_win32_signal(sig,act)
 #endif
 
-CAMLexport int volatile caml_async_signal_mode = 0;
-CAMLexport int volatile caml_pending_signal = 0;
+CAMLexport intnat volatile caml_pending_signals[NSIG];
 CAMLexport int volatile caml_something_to_do = 0;
 int volatile caml_force_major_slice = 0;
 value caml_signal_handlers = 0;
-CAMLexport void (*caml_enter_blocking_section_hook)(void) = NULL;
-CAMLexport void (*caml_leave_blocking_section_hook)(void) = NULL;
 CAMLexport void (* volatile caml_async_action_hook)(void) = NULL;
 
+static void caml_process_pending_signals(void)
+{
+  int signal_num;
+  intnat signal_state;
+
+  for (signal_num = 0; signal_num < NSIG; signal_num++) {
+    Read_and_clear(signal_state, caml_pending_signals[signal_num]);
+    if (signal_state) caml_execute_signal(signal_num, 0);
+  }
+}
+
 void caml_process_event(void)
 {
-  int signal_number;
   void (*async_action)(void);
+
   if (caml_force_major_slice) caml_minor_collection ();
                              /* FIXME should be [caml_check_urgent_gc] */
-  /* If a signal arrives between the following two instructions,
-     it will be lost.  To do: use atomic swap or atomic read-and-clear
-     for processors that support it? */
-  signal_number = caml_pending_signal;
-  caml_pending_signal = 0;
-  if (signal_number) caml_execute_signal(signal_number, 0);
-  /* If an async action is scheduled between the following two instructions,
-     it will be lost. */
-  async_action = caml_async_action_hook;
-  caml_async_action_hook = NULL;
+  caml_process_pending_signals();
+  Read_and_clear(async_action, caml_async_action_hook);
   if (async_action != NULL) (*async_action)();
 }
 
-static int rev_convert_signal_number(int signo);
+static intnat volatile caml_async_signal_mode = 0;
+
+static void caml_enter_blocking_section_default(void)
+{
+  Assert (caml_async_signal_mode == 0);
+  caml_async_signal_mode = 1;
+}
+
+static void caml_leave_blocking_section_default(void)
+{
+  Assert (caml_async_signal_mode == 1);
+  caml_async_signal_mode = 0;
+}
+
+static int caml_try_leave_blocking_section_default(void)
+{
+  intnat res;
+  Read_and_clear(res, caml_async_signal_mode);
+  return res;
+}
+
+CAMLexport void (*caml_enter_blocking_section_hook)(void) =
+   caml_enter_blocking_section_default;
+CAMLexport void (*caml_leave_blocking_section_hook)(void) =
+   caml_leave_blocking_section_default;
+CAMLexport int (*caml_try_leave_blocking_section_hook)(void) =
+   caml_try_leave_blocking_section_default;
+
+CAMLexport int caml_rev_convert_signal_number(int signo);
+
+/* Execute a signal handler immediately */
 
 void caml_execute_signal(int signal_number, int in_signal_handler)
 {
@@ -72,8 +107,9 @@ void caml_execute_signal(int signal_number, int in_signal_handler)
   sigaddset(&sigs, signal_number);
   sigprocmask(SIG_BLOCK, &sigs, &sigs);
 #endif
-  res = caml_callback_exn(Field(caml_signal_handlers, signal_number),
-                          Val_int(rev_convert_signal_number(signal_number)));
+  res = caml_callback_exn(
+           Field(caml_signal_handlers, signal_number),
+           Val_int(caml_rev_convert_signal_number(signal_number)));
 #ifdef POSIX_SIGNALS
   if (! in_signal_handler) {
     /* Restore the original signal mask */
@@ -87,19 +123,27 @@ void caml_execute_signal(int signal_number, int in_signal_handler)
   if (Is_exception_result(res)) caml_raise(Extract_exception(res));
 }
 
+/* Record the delivery of a signal, and arrange so that caml_process_event
+   is called as soon as possible. */
+
+void caml_record_signal(int signal_number)
+{
+  caml_pending_signals[signal_number] = 1;
+  caml_something_to_do = 1;
+}
+
 static void handle_signal(int signal_number)
 {
 #if !defined(POSIX_SIGNALS) && !defined(BSD_SIGNALS)
   signal(signal_number, handle_signal);
 #endif
-  if (caml_async_signal_mode){
-    caml_leave_blocking_section ();
+  if (signal_number < 0 || signal_number >= NSIG) return;
+  if (caml_try_leave_blocking_section_hook()) {
     caml_execute_signal(signal_number, 1);
-    caml_enter_blocking_section ();
+    caml_enter_blocking_section_hook();
   }else{
-    caml_pending_signal = signal_number;
-    caml_something_to_do = 1;
-  }
+    caml_record_signal(signal_number);
+ }
 }
 
 void caml_urge_major_slice (void)
@@ -110,44 +154,26 @@ void caml_urge_major_slice (void)
 
 CAMLexport void caml_enter_blocking_section(void)
 {
-  int temp;
+  int i;
+  intnat pending;
 
   while (1){
-    Assert (!caml_async_signal_mode);
-    /* If a signal arrives between the next two instructions,
-       it will be lost. */
-    temp = caml_pending_signal;   caml_pending_signal = 0;
-    if (temp) caml_execute_signal(temp, 0);
-    caml_async_signal_mode = 1;
-    if (!caml_pending_signal) break;
-    caml_async_signal_mode = 0;
-  }
-  if (caml_enter_blocking_section_hook != NULL){
-    caml_enter_blocking_section_hook();
+    /* Process all pending signals now */
+    caml_process_pending_signals();
+    caml_enter_blocking_section_hook ();
+    /* Check again for pending signals. */
+    pending = 0;
+    for (i = 0; i < NSIG; i++) pending |= caml_pending_signals[i];
+    /* If none, done; otherwise, try again */
+    if (!pending) break;
+    caml_leave_blocking_section_hook ();
   }
 }
 
 CAMLexport void caml_leave_blocking_section(void)
 {
-#ifdef _WIN32
-  int signal_number;
-#endif
-
-  if (caml_leave_blocking_section_hook != NULL){
-    caml_leave_blocking_section_hook();
-  }
-#ifdef _WIN32
-  /* Under Win32, asynchronous signals such as ctrl-C are not processed
-     immediately (see ctrl_handler in win32.c), but simply set
-     [caml_pending_signal] and let the system call run to completion.
-     Hence, test [caml_pending_signal] here and act upon it, before we get
-     a chance to process the result of the system call. */
-  signal_number = caml_pending_signal;
-  caml_pending_signal = 0;
-  if (signal_number) caml_execute_signal(signal_number, 1);
-#endif
-  Assert(caml_async_signal_mode);
-  caml_async_signal_mode = 0;
+  caml_leave_blocking_section_hook ();
+  caml_process_pending_signals();
 }
 
 #ifndef SIGABRT
@@ -228,7 +254,7 @@ CAMLexport int caml_convert_signal_number(int signo)
     return signo;
 }
 
-static int rev_convert_signal_number(int signo)
+CAMLexport int caml_rev_convert_signal_number(int signo)
 {
   int i;
   for (i = 0; i < sizeof(posix_signals) / sizeof(int); i++)
@@ -236,10 +262,6 @@ static int rev_convert_signal_number(int signo)
   return signo;
 }
 
-#ifndef NSIG
-#define NSIG 64
-#endif
-
 CAMLprim value caml_install_signal_handler(value signal_number, value action)
 {
   CAMLparam2 (signal_number, action);
@@ -289,5 +311,6 @@ CAMLprim value caml_install_signal_handler(value signal_number, value action)
     }
     caml_modify(&Field(caml_signal_handlers, sig), Field(action, 0));
   }
+  caml_process_pending_signals();
   CAMLreturn (res);
 }
