@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sys.c,v 1.59 2002/06/18 16:18:21 xleroy Exp $ */
+/* $Id: sys.c,v 1.78 2005/10/13 14:47:05 xleroy Exp $ */
 
 /* Basic system calls */
 
@@ -22,16 +22,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
-#if !macintosh
 #include <sys/types.h>
 #include <sys/stat.h>
-#endif
-#if !macintosh && !_WIN32
+#if !_WIN32
 #include <sys/wait.h>
 #endif
-#if macintosh
-#include "macintosh.h"
-#endif
 #include "config.h"
 #ifdef HAS_UNISTD
 #include <unistd.h>
@@ -47,43 +42,20 @@
 #include "fail.h"
 #include "instruct.h"
 #include "mlvalues.h"
+#include "osdeps.h"
 #include "signals.h"
 #include "stacks.h"
 #include "sys.h"
-#ifdef HAS_UI
-#include "ui.h"
-#endif
 
 #ifndef _WIN32
 extern int errno;
 #endif
 
-#ifdef HAS_STRERROR
-
-#ifndef _WIN32
-extern char * strerror(int);
-#endif
-
-char * error_message(void)
+static char * error_message(void)
 {
   return strerror(errno);
 }
 
-#else
-
-extern int sys_nerr;
-extern char * sys_errlist [];
-
-char * error_message(void)
-{
-  if (errno < 0 || errno >= sys_nerr)
-    return "unknown error";
-  else
-    return sys_errlist[errno];
-}
-
-#endif /* HAS_STRERROR */
-
 #ifndef EAGAIN
 #define EAGAIN (-1)
 #endif
@@ -91,40 +63,37 @@ char * error_message(void)
 #define EWOULDBLOCK (-1)
 #endif
 
-CAMLexport void sys_error(value arg)
+CAMLexport void caml_sys_error(value arg)
 {
   CAMLparam1 (arg);
   char * err;
   CAMLlocal1 (str);
   
   if (errno == EAGAIN || errno == EWOULDBLOCK) {
-    raise_sys_blocked_io();
+    caml_raise_sys_blocked_io();
   } else {
     err = error_message();
     if (arg == NO_ARG) {
-      str = copy_string(err);
+      str = caml_copy_string(err);
     } else {
       int err_len = strlen(err);
-      int arg_len = string_length(arg);
-      str = alloc_string(arg_len + 2 + err_len);
+      int arg_len = caml_string_length(arg);
+      str = caml_alloc_string(arg_len + 2 + err_len);
       memmove(&Byte(str, 0), String_val(arg), arg_len);
       memmove(&Byte(str, arg_len), ": ", 2);
       memmove(&Byte(str, arg_len + 2), err, err_len);
     }
-    raise_sys_error(str);
+    caml_raise_sys_error(str);
   }
+  CAMLnoreturn;
 }
 
-CAMLprim value sys_exit(value retcode)
+CAMLprim value caml_sys_exit(value retcode)
 {
 #ifndef NATIVE_CODE
-  debugger(PROGRAM_EXIT);
+  caml_debugger(PROGRAM_EXIT);
 #endif
-#ifdef HAS_UI
-  ui_exit(Int_val(retcode));
-#else
   exit(Int_val(retcode));
-#endif
   return Val_unit;
 }
 
@@ -147,133 +116,136 @@ static int sys_open_flags[] = {
   O_BINARY, O_TEXT, O_NONBLOCK
 };
 
-CAMLprim value sys_open(value path, value flags, value perm)
+CAMLprim value caml_sys_open(value path, value vflags, value vperm)
 {
-  int fd;
-  fd = open(String_val(path), convert_flag_list(flags, sys_open_flags)
-#if !macintosh
-             , Int_val(perm)
-#endif
-                                       );
-  if (fd == -1) sys_error(path);
+  CAMLparam3(path, vflags, vperm);
+  int fd, flags, perm;
+  char * p;
+
+  p = caml_stat_alloc(caml_string_length(path) + 1);
+  strcpy(p, String_val(path));
+  flags = caml_convert_flag_list(vflags, sys_open_flags);
+  perm = Int_val(vperm);
+  /* open on a named FIFO can block (PR#1533) */
+  caml_enter_blocking_section();
+  fd = open(p, flags, perm);
+  caml_leave_blocking_section();
+  caml_stat_free(p);
+  if (fd == -1) caml_sys_error(path);
 #if defined(F_SETFD) && defined(FD_CLOEXEC)
   fcntl(fd, F_SETFD, FD_CLOEXEC);
 #endif
-  return Val_long(fd);
+  CAMLreturn(Val_long(fd));
 }
 
-CAMLprim value sys_close(value fd)
+CAMLprim value caml_sys_close(value fd)
 {
   close(Int_val(fd));
   return Val_unit;
 }
 
-CAMLprim value sys_file_exists(value name)
+CAMLprim value caml_sys_file_exists(value name)
 {
-#if macintosh
-  int f;
-  f = open (String_val (name), O_RDONLY);
-  if (f == -1) return (Val_bool (0));
-  close (f);
-  return (Val_bool (1));
-#else
   struct stat st;
   return Val_bool(stat(String_val(name), &st) == 0);
-#endif
 }
 
-CAMLprim value sys_remove(value name)
+CAMLprim value caml_sys_remove(value name)
 {
   int ret;
   ret = unlink(String_val(name));
-  if (ret != 0) sys_error(name);
+  if (ret != 0) caml_sys_error(name);
   return Val_unit;
 }
 
-CAMLprim value sys_rename(value oldname, value newname)
+CAMLprim value caml_sys_rename(value oldname, value newname)
 {
   if (rename(String_val(oldname), String_val(newname)) != 0)
-    sys_error(oldname);
+    caml_sys_error(NO_ARG);
   return Val_unit;
 }
 
-CAMLprim value sys_chdir(value dirname)
+CAMLprim value caml_sys_chdir(value dirname)
 {
-  if (chdir(String_val(dirname)) != 0) sys_error(dirname);
+  if (chdir(String_val(dirname)) != 0) caml_sys_error(dirname);
   return Val_unit;
 }
 
-CAMLprim value sys_getcwd(value unit)
+CAMLprim value caml_sys_getcwd(value unit)
 {
   char buff[4096];
 #ifdef HAS_GETCWD
-  if (getcwd(buff, sizeof(buff)) == 0) sys_error(NO_ARG);
+  if (getcwd(buff, sizeof(buff)) == 0) caml_sys_error(NO_ARG);
 #else
-  if (getwd(buff) == 0) sys_error(NO_ARG);
+  if (getwd(buff) == 0) caml_sys_error(NO_ARG);
 #endif /* HAS_GETCWD */
-  return copy_string(buff);
+  return caml_copy_string(buff);
 }
 
-CAMLprim value sys_getenv(value var)
+CAMLprim value caml_sys_getenv(value var)
 {
   char * res;
 
   res = getenv(String_val(var));
-  if (res == 0) raise_not_found();
-  return copy_string(res);
+  if (res == 0) caml_raise_not_found();
+  return caml_copy_string(res);
 }
 
 char * caml_exe_name;
 static char ** caml_main_argv;
 
-CAMLprim value sys_get_argv(value unit)
+CAMLprim value caml_sys_get_argv(value unit)
 {
   CAMLparam0 ();   /* unit is unused */
   CAMLlocal3 (exe_name, argv, res);
-  exe_name = copy_string(caml_exe_name);
-  argv = copy_string_array((char const **) caml_main_argv);
-  res = alloc_small(2, 0);
+  exe_name = caml_copy_string(caml_exe_name);
+  argv = caml_copy_string_array((char const **) caml_main_argv);
+  res = caml_alloc_small(2, 0);
   Field(res, 0) = exe_name;
   Field(res, 1) = argv;
   CAMLreturn(res);
 }
 
-void sys_init(char * exe_name, char **argv)
+void caml_sys_init(char * exe_name, char **argv)
 {
   caml_exe_name = exe_name;
   caml_main_argv = argv;
 }
 
+#ifdef _WIN32
+#define WIFEXITED(status) 1
+#define WEXITSTATUS(status) (status)
+#else
 #if !(defined(WIFEXITED) && defined(WEXITSTATUS))
 /* Assume old-style V7 status word */
 #define WIFEXITED(status) (((status) & 0xFF) == 0)
 #define WEXITSTATUS(status) (((status) >> 8) & 0xFF)
 #endif
-
-#ifdef _WIN32
-extern int win32_system(char * command);
 #endif
 
-CAMLprim value sys_system_command(value command)
+CAMLprim value caml_sys_system_command(value command)
 {
+  CAMLparam1 (command);
   int status, retcode;
+  char *buf;
+  intnat len;
   
-  enter_blocking_section ();
-#ifndef _WIN32
-  status = system(String_val(command));
+  len = caml_string_length (command);
+  buf = caml_stat_alloc (len + 1);
+  memmove (buf, String_val (command), len + 1);
+  caml_enter_blocking_section ();
+  status = system(buf);
+  caml_leave_blocking_section ();
+  caml_stat_free(buf);
+  if (status == -1) caml_sys_error(command);
   if (WIFEXITED(status))
     retcode = WEXITSTATUS(status);
   else
     retcode = 255;
-#else
-  status = retcode = win32_system(String_val(command));
-#endif
-  leave_blocking_section ();
-  if (status == -1) sys_error(command);
-  return Val_int(retcode);
+  CAMLreturn (Val_int(retcode));
 }
 
-CAMLprim value sys_time(value unit)
+CAMLprim value caml_sys_time(value unit)
 {
 #ifdef HAS_TIMES
 #ifndef CLK_TCK
@@ -285,16 +257,16 @@ CAMLprim value sys_time(value unit)
 #endif
   struct tms t;
   times(&t);
-  return copy_double((double)(t.tms_utime + t.tms_stime) / CLK_TCK);
+  return caml_copy_double((double)(t.tms_utime + t.tms_stime) / CLK_TCK);
 #else
   /* clock() is standard ANSI C */
-  return copy_double((double)clock() / CLOCKS_PER_SEC);
+  return caml_copy_double((double)clock() / CLOCKS_PER_SEC);
 #endif
 }
 
-CAMLprim value sys_random_seed (value unit)
+CAMLprim value caml_sys_random_seed (value unit)
 {
-  long seed;
+  intnat seed;
 #ifdef HAS_GETTIMEOFDAY
   struct timeval tv;
   gettimeofday(&tv, NULL);
@@ -308,15 +280,28 @@ CAMLprim value sys_random_seed (value unit)
   return Val_long(seed);
 }
 
-CAMLprim value sys_get_config(value unit)
+CAMLprim value caml_sys_get_config(value unit)
 {
   CAMLparam0 ();   /* unit is unused */
   CAMLlocal2 (result, ostype);
 
-  ostype = copy_string(OCAML_OS_TYPE);
-  result = alloc_small (2, 0);
+  ostype = caml_copy_string(OCAML_OS_TYPE);
+  result = caml_alloc_small (2, 0);
   Field(result, 0) = ostype;
   Field(result, 1) = Val_long (8 * sizeof(value));
   CAMLreturn (result);
 }
 
+CAMLprim value caml_sys_read_directory(value path)
+{
+  CAMLparam1(path);
+  CAMLlocal1(result);
+  struct ext_table tbl;
+
+  caml_ext_table_init(&tbl, 50);
+  if (caml_read_directory(String_val(path), &tbl) == -1) caml_sys_error(path);
+  caml_ext_table_add(&tbl, NULL);
+  result = caml_copy_string_array((char const **) tbl.contents);
+  caml_ext_table_free(&tbl, 1);
+  CAMLreturn(result);
+}
