@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_r.ml,v 1.59.2.3 2005/06/02 10:40:32 mauny Exp $ *)
+(* $Id: pa_r.ml,v 1.64 2005/06/29 04:11:26 garrigue Exp $ *)
 
 open Stdpp;
 open Pcaml;
@@ -72,13 +72,13 @@ value o2b =
   | None -> False ]
 ;
 
-value mksequence loc =
+value mksequence _loc =
   fun
   [ [e] -> e
   | el -> <:expr< do { $list:el$ } >> ]
 ;
 
-value mkmatchcase loc p aso w e =
+value mkmatchcase _loc p aso w e =
   let p =
     match aso with
     [ Some p2 -> <:patt< ($p$ as $p2$) >>
@@ -93,7 +93,7 @@ value neg_string n =
   else "-" ^ n
 ;
 
-value mkumin loc f arg =
+value mkumin _loc f arg =
   match arg with
   [ <:expr< $int:n$ >> -> <:expr< $int:neg_string n$ >>
   | MLast.ExInt32 loc n -> MLast.ExInt32 loc (neg_string n)
@@ -105,7 +105,7 @@ value mkumin loc f arg =
       <:expr< $lid:f$ $arg$ >> ]
 ;
 
-value mklistexp loc last =
+value mklistexp _loc last =
   loop True where rec loop top =
     fun
     [ [] ->
@@ -113,13 +113,13 @@ value mklistexp loc last =
         [ Some e -> e
         | None -> <:expr< [] >> ]
     | [e1 :: el] ->
-        let loc =
-          if top then loc else (fst (MLast.loc_of_expr e1), snd loc)
+        let _loc =
+          if top then _loc else (fst (MLast.loc_of_expr e1), snd _loc)
         in
         <:expr< [$e1$ :: $loop False el$] >> ]
 ;
 
-value mklistpat loc last =
+value mklistpat _loc last =
   loop True where rec loop top =
     fun
     [ [] ->
@@ -127,25 +127,26 @@ value mklistpat loc last =
         [ Some p -> p
         | None -> <:patt< [] >> ]
     | [p1 :: pl] ->
-        let loc =
-          if top then loc else (fst (MLast.loc_of_patt p1), snd loc)
+        let _loc =
+          if top then _loc else (fst (MLast.loc_of_patt p1), snd _loc)
         in
         <:patt< [$p1$ :: $loop False pl$] >> ]
 ;
 
-value mkexprident loc i j =
-  let rec loop m =
-    fun
-    [ <:expr< $x$ . $y$ >> -> loop <:expr< $m$ . $x$ >> y
-    | e -> <:expr< $m$ . $e$ >> ]
+value mkexprident _loc ids = match ids with
+  [ [] -> Stdpp.raise_with_loc _loc (Stream.Error "illegal long identifier")
+  | [ id :: ids ] ->
+      let rec loop m = fun
+        [ [ id :: ids ] -> loop <:expr< $m$ . $id$ >> ids
+        | [] -> m ]
   in
-  loop <:expr< $uid:i$ >> j
+  loop id ids ]
 ;
 
-value mkassert loc e =
+value mkassert _loc e =
   match e with
-  [ <:expr< False >> -> MLast.ExAsf loc
-  | _ -> MLast.ExAsr loc e ]
+  [ <:expr< False >> -> MLast.ExAsf _loc
+  | _ -> MLast.ExAsr _loc e ]
 ;
 
 value append_elem el e = el @ [e];
@@ -223,7 +224,7 @@ EXTEND
       | "module"; i = UIDENT; mb = module_binding ->
           <:str_item< module $i$ = $mb$ >>
       | "module"; "rec"; nmtmes = LIST1 module_rec_binding SEP "and" ->
-          MLast.StRecMod loc nmtmes
+          MLast.StRecMod _loc nmtmes
       | "module"; "type"; i = UIDENT; "="; mt = module_type ->
           <:str_item< module type $i$ = $mt$ >>
       | "open"; i = mod_ident -> <:str_item< open $i$ >>
@@ -276,7 +277,7 @@ EXTEND
       | "module"; i = UIDENT; mt = module_declaration ->
           <:sig_item< module $i$ : $mt$ >>
       | "module"; "rec"; mds = LIST1 module_rec_declaration SEP "and" ->
-          MLast.SgRecMod loc mds
+          MLast.SgRecMod _loc mds
       | "module"; "type"; i = UIDENT; "="; mt = module_type ->
           <:sig_item< module type $i$ = $mt$ >>
       | "open"; i = mod_ident -> <:sig_item< open $i$ >>
@@ -320,7 +321,7 @@ EXTEND
           <:expr< try $e$ with $p1$ -> $e1$ >>
       | "if"; e1 = SELF; "then"; e2 = SELF; "else"; e3 = SELF ->
           <:expr< if $e1$ then $e2$ else $e3$ >>
-      | "do"; "{"; seq = sequence; "}" -> mksequence loc seq
+      | "do"; "{"; seq = sequence; "}" -> mksequence _loc seq
       | "for"; i = LIDENT; "="; e1 = SELF; df = direction_flag; e2 = SELF;
         "do"; "{"; seq = sequence; "}" ->
           <:expr< for $i$ = $e1$ $to:df$ $e2$ do { $list:seq$ } >>
@@ -328,7 +329,7 @@ EXTEND
           <:expr< while $e$ do { $list:seq$ } >>
       | "object"; cspo = OPT class_self_patt; cf = class_structure; "end" ->
           (* <:expr< object $opt:cspo$ $list:cf$ end >> *)
-          MLast.ExObj loc cspo cf ]
+          MLast.ExObj _loc cspo cf ]
     | "where"
       [ e = SELF; "where"; rf = OPT "rec"; lb = let_binding ->
           <:expr< let $opt:o2b rf$ $list:[lb]$ in $e$ >> ]
@@ -370,11 +371,11 @@ EXTEND
       | e1 = SELF; "lsl"; e2 = SELF -> <:expr< $e1$ lsl $e2$ >>
       | e1 = SELF; "lsr"; e2 = SELF -> <:expr< $e1$ lsr $e2$ >> ]
     | "unary minus" NONA
-      [ "-"; e = SELF -> mkumin loc "-" e
-      | "-."; e = SELF -> mkumin loc "-." e ]
+      [ "-"; e = SELF -> mkumin _loc "-" e
+      | "-."; e = SELF -> mkumin _loc "-." e ]
     | "apply" LEFTA
       [ e1 = SELF; e2 = SELF -> <:expr< $e1$ $e2$ >>
-      | "assert"; e = SELF -> mkassert loc e
+      | "assert"; e = SELF -> mkassert _loc e
       | "lazy"; e = SELF -> <:expr< lazy ($e$) >> ]
     | "." LEFTA
       [ e1 = SELF; "."; "("; e2 = SELF; ")" -> <:expr< $e1$ .( $e2$ ) >>
@@ -385,16 +386,16 @@ EXTEND
       | "~-."; e = SELF -> <:expr< ~-. $e$ >> ]
     | "simple"
       [ s = INT -> <:expr< $int:s$ >>
-      | s = INT32 -> MLast.ExInt32 loc s
-      | s = INT64 -> MLast.ExInt64 loc s
-      | s = NATIVEINT -> MLast.ExNativeInt loc s
+      | s = INT32 -> MLast.ExInt32 _loc s
+      | s = INT64 -> MLast.ExInt64 _loc s
+      | s = NATIVEINT -> MLast.ExNativeInt _loc s
       | s = FLOAT -> <:expr< $flo:s$ >>
       | s = STRING -> <:expr< $str:s$ >>
       | s = CHAR -> <:expr< $chr:s$ >>
-      | i = expr_ident -> i
+      | ids = expr_ident -> mkexprident _loc ids
       | "["; "]" -> <:expr< [] >>
       | "["; el = LIST1 expr SEP ";"; last = cons_expr_opt; "]" ->
-          mklistexp loc last el
+          mklistexp _loc last el
       | "[|"; el = LIST0 expr SEP ";"; "|]" -> <:expr< [| $list:el$ |] >>
       | "{"; lel = LIST1 label_expr SEP ";"; "}" -> <:expr< { $list:lel$ } >>
       | "{"; "("; e = SELF; ")"; "with"; lel = LIST1 label_expr SEP ";"; "}"
@@ -415,7 +416,7 @@ EXTEND
   sequence:
     [ [ "let"; rf = OPT "rec"; l = LIST1 let_binding SEP "and"; [ "in" | ";" ];
         el = SELF ->
-          [<:expr< let $opt:o2b rf$ $list:l$ in $mksequence loc el$ >>]
+          [<:expr< let $opt:o2b rf$ $list:l$ in $mksequence _loc el$ >>]
       | e = expr; ";"; el = SELF -> [e :: el]
       | e = expr; ";" -> [e]
       | e = expr -> [e] ] ]
@@ -432,7 +433,7 @@ EXTEND
   ;
   match_case:
     [ [ p = patt; aso = as_patt_opt; w = when_expr_opt; "->"; e = expr ->
-          mkmatchcase loc p aso w e ] ]
+          mkmatchcase _loc p aso w e ] ]
   ;
   as_patt_opt:
     [ [ "as"; p = patt -> Some p
@@ -447,9 +448,9 @@ EXTEND
   ;
   expr_ident:
     [ RIGHTA
-      [ i = LIDENT -> <:expr< $lid:i$ >>
-      | i = UIDENT -> <:expr< $uid:i$ >>
-      | i = UIDENT; "."; j = SELF -> mkexprident loc i j ] ]
+      [ i = LIDENT -> [ <:expr< $lid:i$ >> ]
+      | i = UIDENT -> [ <:expr< $uid:i$ >> ]
+      | i = UIDENT; "."; j = SELF -> [ <:expr< $uid:i$ >> :: j ] ] ]
   ;
   fun_def:
     [ RIGHTA
@@ -469,20 +470,20 @@ EXTEND
       [ s = LIDENT -> <:patt< $lid:s$ >>
       | s = UIDENT -> <:patt< $uid:s$ >>
       | s = INT -> <:patt< $int:s$ >>
-      | s = INT32 -> MLast.PaInt32 loc s
-      | s = INT64 -> MLast.PaInt64 loc s
-      | s = NATIVEINT -> MLast.PaNativeInt loc s
+      | s = INT32 -> MLast.PaInt32 _loc s
+      | s = INT64 -> MLast.PaInt64 _loc s
+      | s = NATIVEINT -> MLast.PaNativeInt _loc s
       | s = FLOAT -> <:patt< $flo:s$ >>
       | s = STRING -> <:patt< $str:s$ >>
       | s = CHAR -> <:patt< $chr:s$ >>
-      | "-"; s = INT -> MLast.PaInt loc (neg_string s)
-      | "-"; s = INT32 -> MLast.PaInt32 loc (neg_string s)
-      | "-"; s = INT64 -> MLast.PaInt64 loc (neg_string s)
-      | "-"; s = NATIVEINT -> MLast.PaNativeInt loc (neg_string s)
+      | "-"; s = INT -> MLast.PaInt _loc (neg_string s)
+      | "-"; s = INT32 -> MLast.PaInt32 _loc (neg_string s)
+      | "-"; s = INT64 -> MLast.PaInt64 _loc (neg_string s)
+      | "-"; s = NATIVEINT -> MLast.PaNativeInt _loc (neg_string s)
       | "-"; s = FLOAT -> <:patt< $flo:neg_string s$ >>
       | "["; "]" -> <:patt< [] >>
       | "["; pl = LIST1 patt SEP ";"; last = cons_patt_opt; "]" ->
-          mklistpat loc last pl
+          mklistpat _loc last pl
       | "[|"; pl = LIST0 patt SEP ";"; "|]" -> <:patt< [| $list:pl$ |] >>
       | "{"; lpl = LIST1 label_patt SEP ";"; "}" -> <:patt< { $list:lpl$ } >>
       | "("; ")" -> <:patt< () >>
@@ -527,7 +528,7 @@ EXTEND
           (n, tpl, tk, cl) ] ]
   ;
   type_patt:
-    [ [ n = LIDENT -> (loc, n) ] ]
+    [ [ n = LIDENT -> (_loc, n) ] ]
   ;
   constrain:
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp -> (t1, t2) ] ]
@@ -540,7 +541,9 @@ EXTEND
   ctyp:
     [ LEFTA
       [ t1 = SELF; "=="; t2 = SELF -> <:ctyp< $t1$ == $t2$ >> ]
-    | LEFTA
+    | NONA
+      [ "private"; t = ctyp LEVEL "alias" -> <:ctyp< private $t$ >> ]
+    | "alias" LEFTA
       [ t1 = SELF; "as"; t2 = SELF -> <:ctyp< $t1$ as $t2$ >> ]
     | LEFTA
       [ "!"; pl = LIST1 typevar; "."; t = ctyp ->
@@ -564,22 +567,20 @@ EXTEND
       | "("; t = SELF; "*"; tl = LIST1 ctyp SEP "*"; ")" ->
           <:ctyp< ( $list:[t::tl]$ ) >>
       | "("; t = SELF; ")" -> <:ctyp< $t$ >>
-      | "private"; "["; cdl = LIST0 constructor_declaration SEP "|"; "]" ->
-          <:ctyp< private [ $list:cdl$ ] >>
-      | "private"; "{"; ldl = LIST1 label_declaration SEP ";"; "}" ->
-          <:ctyp< private { $list:ldl$ } >>
       | "["; cdl = LIST0 constructor_declaration SEP "|"; "]" ->
           <:ctyp< [ $list:cdl$ ] >>
+          (* MLast.TySum _loc cdl *)
       | "{"; ldl = LIST1 label_declaration SEP ";"; "}" ->
-          <:ctyp< { $list:ldl$ } >> ] ]
+          <:ctyp< { $list:ldl$ } >>
+          (* MLast.TyRec _loc ldl *) ] ]
   ;
   constructor_declaration:
-    [ [ ci = UIDENT; "of"; cal = LIST1 ctyp SEP "and" -> (loc, ci, cal)
-      | ci = UIDENT -> (loc, ci, []) ] ]
+    [ [ ci = UIDENT; "of"; cal = LIST1 ctyp SEP "and" -> (_loc, ci, cal)
+      | ci = UIDENT -> (_loc, ci, []) ] ]
   ;
   label_declaration:
     [ [ i = LIDENT; ":"; mf = OPT "mutable"; t = ctyp ->
-          (loc, i, o2b mf, t) ] ]
+          (_loc, i, o2b mf, t) ] ]
   ;
   ident:
     [ [ i = LIDENT -> i
@@ -607,7 +608,7 @@ EXTEND
   class_declaration:
     [ [ vf = OPT "virtual"; i = LIDENT; ctp = class_type_parameters;
         cfb = class_fun_binding ->
-          {MLast.ciLoc = loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
+          {MLast.ciLoc = _loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
            MLast.ciNam = i; MLast.ciExp = cfb} ] ]
   ;
   class_fun_binding:
@@ -617,8 +618,8 @@ EXTEND
       | p = ipatt; cfb = SELF -> <:class_expr< fun $p$ -> $cfb$ >> ] ]
   ;
   class_type_parameters:
-    [ [ -> (loc, [])
-      | "["; tpl = LIST1 type_parameter SEP ","; "]" -> (loc, tpl) ] ]
+    [ [ -> (_loc, [])
+      | "["; tpl = LIST1 type_parameter SEP ","; "]" -> (_loc, tpl) ] ]
   ;
   class_fun_def:
     [ [ p = ipatt; ce = SELF -> <:class_expr< fun $p$ -> $ce$ >>
@@ -712,13 +713,13 @@ EXTEND
   class_description:
     [ [ vf = OPT "virtual"; n = LIDENT; ctp = class_type_parameters; ":";
         ct = class_type ->
-          {MLast.ciLoc = loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
+          {MLast.ciLoc = _loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
            MLast.ciNam = n; MLast.ciExp = ct} ] ]
   ;
   class_type_declaration:
     [ [ vf = OPT "virtual"; n = LIDENT; ctp = class_type_parameters; "=";
         cs = class_type ->
-          {MLast.ciLoc = loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
+          {MLast.ciLoc = _loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
            MLast.ciNam = n; MLast.ciExp = cs} ] ]
   ;
   expr: LEVEL "apply"
@@ -857,7 +858,7 @@ EXTEND
           <:ctyp< [ < $list:rfl$ > $list:ntl$ ] >> ] ]
   ;
   warning_variant:
-    [ [ -> warn_variant loc ] ]
+    [ [ -> warn_variant _loc ] ]
   ;
   (* Compatibility old syntax of sequences *)
   expr: LEVEL "top"
@@ -872,7 +873,7 @@ EXTEND
           <:expr< while $e$ do { $list:seq$ } >> ] ]
   ;
   warning_sequence:
-    [ [ -> warn_sequence loc ] ]
+    [ [ -> warn_sequence _loc ] ]
   ;
 END;
 
@@ -880,21 +881,21 @@ EXTEND
   GLOBAL: interf implem use_file top_phrase expr patt;
   interf:
     [ [ "#"; n = LIDENT; dp = OPT expr; ";" ->
-          ([(<:sig_item< # $n$ $opt:dp$ >>, loc)], True)
+          ([(<:sig_item< # $n$ $opt:dp$ >>, _loc)], True)
       | si = sig_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
       | EOI -> ([], False) ] ]
   ;
   sig_item_semi:
-    [ [ si = sig_item; ";" -> (si, loc) ] ]
+    [ [ si = sig_item; ";" -> (si, _loc) ] ]
   ;
   implem:
     [ [ "#"; n = LIDENT; dp = OPT expr; ";" ->
-          ([(<:str_item< # $n$ $opt:dp$ >>, loc)], True)
+          ([(<:str_item< # $n$ $opt:dp$ >>, _loc)], True)
       | si = str_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
       | EOI -> ([], False) ] ]
   ;
   str_item_semi:
-    [ [ si = str_item; ";" -> (si, loc) ] ]
+    [ [ si = str_item; ";" -> (si, _loc) ] ]
   ;
   top_phrase:
     [ [ ph = phrase -> Some ph
@@ -922,7 +923,7 @@ EXTEND
             with
             [ Not_found | Failure _ -> ({(Lexing.dummy_pos) with Lexing.pos_cnum = 0}, x) ]
           in
-          Pcaml.handle_expr_locate loc x
+          Pcaml.handle_expr_locate _loc x
       | x = QUOTATION ->
           let x =
             try
@@ -932,7 +933,7 @@ EXTEND
             with
             [ Not_found -> ("", x) ]
           in
-          Pcaml.handle_expr_quotation loc x ] ]
+          Pcaml.handle_expr_quotation _loc x ] ]
   ;
   patt: LEVEL "simple"
     [ [ x = LOCATE ->
@@ -945,7 +946,7 @@ EXTEND
             with
             [ Not_found | Failure _ -> ({(Lexing.dummy_pos) with Lexing.pos_cnum = 0}, x) ]
           in
-          Pcaml.handle_patt_locate loc x
+          Pcaml.handle_patt_locate _loc x
       | x = QUOTATION ->
           let x =
             try
@@ -955,6 +956,6 @@ EXTEND
             with
             [ Not_found -> ("", x) ]
           in
-          Pcaml.handle_patt_quotation loc x ] ]
+          Pcaml.handle_patt_quotation _loc x ] ]
   ;
 END;
