@@ -9,7 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odoc_sig.ml,v 1.30.2.4 2005/06/23 14:23:27 guesdon Exp $ *)
+(* $Id: odoc_sig.ml,v 1.36 2005/08/13 20:59:37 doligez Exp $ *)
 
 (** Analysis of interface files. *)
 
@@ -170,96 +170,43 @@ module Analyser =
 
     let merge_infos = Odoc_merge.merge_info_opt Odoc_types.all_merge_options
 
-    let name_comment_from_type_kind pos_start pos_end pos_limit tk =
+    let name_comment_from_type_kind pos_end pos_limit tk =
       match tk with
-        Parsetree.Ptype_abstract ->
+        Parsetree.Ptype_abstract | Parsetree.Ptype_private ->
           (0, [])
       | Parsetree.Ptype_variant (cons_core_type_list_list, _) ->
-          (*of (string * core_type list) list *)
-          let rec f acc last_pos cons_core_type_list_list =
+          let rec f acc cons_core_type_list_list =
             match cons_core_type_list_list with
               [] ->
                 (0, acc)
-            | (name, core_type_list) :: [] ->
-                let pos = Str.search_forward (Str.regexp_string name) !file last_pos in
-                let s = get_string_of_file pos_end pos_limit in
+            | (name, core_type_list, loc) :: [] ->
+                let s = get_string_of_file
+		    loc.Location.loc_end.Lexing.pos_cnum
+		    pos_limit
+		in
                 let (len, comment_opt) =  My_ir.just_after_special !file_name s in
                 (len, acc @ [ (name, comment_opt) ])
-
-            | (name, core_type_list) :: (name2, core_type_list2) :: q ->
-                match (List.rev core_type_list, core_type_list2) with
-                  ([], []) ->
-                    let pos = Str.search_forward (Str.regexp_string name) !file last_pos in
-                    let pos' = pos + (String.length name) in
-                    let pos2 =
-		      try Str.search_forward
-			  (Str.regexp ("|[ \n\t\r]*"^name2)) !file pos'
-		      with Not_found ->
-			failwith (Odoc_messages.misplaced_comment !file_name pos')
-		    in
-                    let s = get_string_of_file pos' pos2 in
-                    let (_,comment_opt) =  My_ir.just_after_special !file_name  s in
-                    f (acc @ [name, comment_opt]) pos2 ((name2, core_type_list2) :: q)
-
-                | ([], (ct2 :: _)) ->
-                    let pos = Str.search_forward (Str.regexp_string name) !file last_pos in
-                    let pos' = pos + (String.length name) in
-                    let pos2 = ct2.Parsetree.ptyp_loc.Location.loc_start.Lexing.pos_cnum in
-                    let pos2' =
-		      try Str.search_backward
-			  (Str.regexp ("|[ \n\t\r]*"^name2)) !file pos2
-		      with Not_found ->
-			failwith (Odoc_messages.misplaced_comment !file_name pos')
-		    in
-                    let s = get_string_of_file pos' pos2' in
-                    let (_,comment_opt) =  My_ir.just_after_special !file_name  s in
-                    f (acc @ [name, comment_opt]) pos2' ((name2, core_type_list2) :: q)
-
-                | ((ct :: _), []) ->
-                    let pos = ct.Parsetree.ptyp_loc.Location.loc_end.Lexing.pos_cnum in
-                    let pos2 =
-		      try
-			Str.search_forward
-			  (Str.regexp ("|[ \n\t\r]*"^name2))
-			  !file pos
-		      with Not_found ->
-			failwith (Odoc_messages.misplaced_comment !file_name pos)
-		    in
-                    let s = get_string_of_file pos pos2 in
-                    let (_,comment_opt) =  My_ir.just_after_special !file_name  s in
-                    let new_pos_end =
-                      match comment_opt with
-                        None -> ct.Parsetree.ptyp_loc.Location.loc_end.Lexing.pos_cnum
-                       | Some _ -> Str.search_forward (Str.regexp "*)") !file pos
-                    in
-                    f (acc @ [name, comment_opt]) new_pos_end ((name2, core_type_list2) :: q)
-
-		| ((ct:: _), (ct2 :: _)) ->
-		    let pos = ct.Parsetree.ptyp_loc.Location.loc_end.Lexing.pos_cnum in
-                    let pos2 = ct2.Parsetree.ptyp_loc.Location.loc_start.Lexing.pos_cnum in
-                    let pos2' =
-		      try Str.search_backward
-			  (Str.regexp ("|[ \n\t\r]*"^name2)) !file pos2
-		      with Not_found ->
-			failwith (Odoc_messages.misplaced_comment !file_name pos)
-		    in
-                    let s = get_string_of_file pos pos2' in
-                    let (_,comment_opt) =  My_ir.just_after_special !file_name  s in
-                    f (acc @ [name, comment_opt]) pos2' ((name2, core_type_list2) :: q)
-
+            | (name, core_type_list, loc) :: (name2, core_type_list2, loc2)
+              :: q ->
+		let pos_end_first = loc.Location.loc_end.Lexing.pos_cnum in
+		let pos_start_second = loc2.Location.loc_start.Lexing.pos_cnum in
+		let s = get_string_of_file pos_end_first pos_start_second in
+		let (_,comment_opt) = My_ir.just_after_special !file_name  s in
+		f (acc @ [name, comment_opt])
+                  ((name2, core_type_list2, loc2) :: q)
           in
-          f [] pos_start cons_core_type_list_list
+          f [] cons_core_type_list_list
 
       | Parsetree.Ptype_record (name_mutable_type_list, _) (* of (string * mutable_flag * core_type) list*) ->
           let rec f = function
               [] ->
                 []
-            | (name, _, ct) :: [] ->
+            | (name, _, ct, xxloc) :: [] ->
                 let pos = ct.Parsetree.ptyp_loc.Location.loc_end.Lexing.pos_cnum in
                 let s = get_string_of_file pos pos_end in
                 let (_,comment_opt) =  My_ir.just_after_special !file_name s in
                 [name, comment_opt]
-            | (name,_,ct) :: ((name2,_,ct2) as ele2) :: q ->
+            | (name,_,ct,xxloc) :: ((name2,_,ct2,xxloc2) as ele2) :: q ->
                 let pos = ct.Parsetree.ptyp_loc.Location.loc_end.Lexing.pos_cnum in
                 let pos2 = ct2.Parsetree.ptyp_loc.Location.loc_start.Lexing.pos_cnum in
                 let s = get_string_of_file pos pos2 in
@@ -633,7 +580,6 @@ module Analyser =
                   in
                   let (maybe_more, name_comment_list) =
                     name_comment_from_type_kind
-                      type_decl.Parsetree.ptype_loc.Location.loc_start.Lexing.pos_cnum
                       type_decl.Parsetree.ptype_loc.Location.loc_end.Lexing.pos_cnum
                       pos_limit2
                       type_decl.Parsetree.ptype_kind
@@ -1375,5 +1321,3 @@ module Analyser =
       }
 
     end
-
-(* eof $Id: odoc_sig.ml,v 1.30.2.4 2005/06/23 14:23:27 guesdon Exp $ *)
