@@ -10,10 +10,12 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: syntaxerr.mli,v 1.3 1999/11/17 18:58:19 xleroy Exp $ *)
+(* $Id: syntaxerr.mli,v 1.4 2000/03/06 22:11:40 weis Exp $ *)
 
 (* Auxiliary type for reporting syntax errors *)
 
+open Format
+
 type error =
     Unclosed of Location.t * string * Location.t * string
   | Other of Location.t
@@ -21,4 +23,4 @@ type error =
 exception Error of error
 exception Escape_error
 
-val report_error: error -> unit
+val report_error: formatter -> error -> unit
