@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: filename.mli,v 1.24 2001/12/07 13:40:50 xleroy Exp $ *)
+(* $Id: filename.mli,v 1.33 2005/10/25 18:34:07 doligez Exp $ *)
 
 (** Operations on file names. *)
 
@@ -48,41 +48,52 @@ val chop_suffix : string -> string -> string
 
 val chop_extension : string -> string
 (** Return the given file name without its extension. The extension
-   is the shortest suffix starting with a period, [.xyz] for instance.
+   is the shortest suffix starting with a period and not including
+   a directory separator, [.xyz] for instance.
 
    Raise [Invalid_argument] if the given name does not contain
-   a period. *)
+   an extension. *)
 
 val basename : string -> string
 (** Split a file name into directory name / base file name.
-   {!Filename.concat} [(]{!Filename.dirname}[ name) (]{!Filename.basename}[ name)]
-   returns a file name which is equivalent to [name]. Moreover, after setting the
-   current directory to {!Filename.dirname}[ name] (with {!Sys.chdir}),
-   references to {!Filename.basename}[ name] (which is a relative file name)
-   designate the same file as [name] before the call to {!Sys.chdir}. *)
+   [concat (dirname name) (basename name)] returns a file name
+   which is equivalent to [name]. Moreover, after setting the
+   current directory to [dirname name] (with {!Sys.chdir}),
+   references to [basename name] (which is a relative file name)
+   designate the same file as [name] before the call to {!Sys.chdir}.
+
+   The result is not specified if the argument is not a valid file name
+   (for example, under Unix if there is a NUL character in the string). *)
 
 val dirname : string -> string
-(** See {!Filename.dirname}. *)
+(** See {!Filename.basename}. *)
 
 val temp_file : string -> string -> string
 (** [temp_file prefix suffix] returns the name of a
    fresh temporary file in the temporary directory.
    The base name of the temporary file is formed by concatenating
    [prefix], then a suitably chosen integer number, then [suffix].
-   The temporary file is created empty, and is guaranteed to be
-   different from any other file that existed when [temp_file]
-   was called.
+   The temporary file is created empty, with permissions [0o600]
+   (readable and writable only by the file owner).  The file is
+   guaranteed to be different from any other file that existed when
+   [temp_file] was called.
    Under Unix, the temporary directory is [/tmp] by default; if set,
    the value of the environment variable [TMPDIR] is used instead.
    Under Windows, the name of the temporary directory is the
-   value of the environment variable [TEMP],
-   or [C:\temp] by default.
-   Under MacOS, the name of the temporary directory is given
-   by the environment variable [TempFolder]; if not set,
-   temporary files are created in the current directory. *)
+   value of the environment variable [TEMP], or [C:\temp] by default. *)
+
+val open_temp_file :
+      ?mode: open_flag list -> string -> string -> string * out_channel
+(** Same as {!Filename.temp_file}, but returns both the name of a fresh
+   temporary file, and an output channel opened (atomically) on
+   this file.  This function is more secure than [temp_file]: there
+   is no risk that the temporary file will be modified (e.g. replaced
+   by a symbolic link) before the program opens it.  The optional argument
+   [mode] is a list of additional flags to control the opening of the file.
+   It can contain one or several of [Open_append], [Open_binary],
+   and [Open_text].  The default is [[Open_text]] (open in text mode). *)
 
 val quote : string -> string
 (** Return a quoted version of a file name, suitable for use as
-   one argument in a shell command line, escaping any shell
+   one argument in a shell command line, escaping all shell
    meta-characters. *)
-
