@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: headernt.c,v 1.16 2001/12/07 13:40:52 xleroy Exp $ */
+/* $Id: headernt.c,v 1.19 2002/07/23 14:12:01 doligez Exp $ */
 
 #define STRICT
 #define WIN32_LEAN_AND_MEAN
@@ -20,9 +20,11 @@
 #include "../byterun/mlvalues.h"
 #include "../byterun/exec.h"
 
+#ifndef __MINGW32__
 #pragma comment(linker , "/entry:headerentry")
 #pragma comment(linker , "/subsystem:console")
 #pragma comment(lib , "kernel32")
+#endif
 
 char * default_runtime_name = "ocamlrun";
 
@@ -75,7 +77,7 @@ static __inline char * read_runtime_path(HANDLE h)
 static BOOL WINAPI ctrl_handler(DWORD event)
 {
   if (event == CTRL_C_EVENT || event == CTRL_BREAK_EVENT)
-    return TRUE;		/* pretend we've handled them */
+    return TRUE;                /* pretend we've handled them */
   else
     return FALSE;
 }
@@ -135,7 +137,11 @@ static __inline void __declspec(noreturn) run_runtime(char * runtime,
 #endif
 }
 
+#ifdef __MINGW32__
+int main()
+#else
 void __declspec(noreturn) __cdecl headerentry()
+#endif
 {
   char truename[MAX_PATH];
   char * cmdline = GetCommandLine();
@@ -163,4 +169,7 @@ void __declspec(noreturn) __cdecl headerentry()
 #if _MSC_VER >= 1200
     __assume(0); /* Not reached */
 #endif
+#ifdef __MINGW32__
+    return 0;
+#endif
 }
