@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: scanf.ml,v 1.49.2.2 2004/07/02 22:24:24 weis Exp $ *)
+(* $Id: scanf.ml,v 1.63 2005/09/20 21:42:44 weis Exp $ *)
 
 (* The run-time library for scanners. *)
 
@@ -21,75 +21,83 @@ module type SCANNING = sig
 type scanbuf;;
 
 val stdib : scanbuf;;
-(** The scanning buffer reading from [stdin].
+(* The scanning buffer reading from [stdin].
     [stdib] is equivalent to [Scanning.from_channel stdin]. *)
 
-val next_char : scanbuf -> unit;;
-(** [Scanning.next_char scanbuf] advance the scanning buffer for
+val next_char : scanbuf -> char;;
+(* [Scanning.next_char ib] advance the scanning buffer for
     one character.
     If no more character can be read, sets a end of file condition and
     returns '\000'. *)
 
-val peek_char : scanbuf -> char;;
-(** [Scanning.peek_char scanbuf] returns the current char available in
-    the buffer. *)
+val invalidate_current_char : scanbuf -> unit;;
+(* [Scanning.invalidate_current_char ib] mark the current_char as already
+    scanned. *)
 
-val cautious_peek_char : scanbuf -> char;;
-(** [Scanning.cautious_peek_char scanbuf] returns the current char
-    available in the buffer or tries to read one if none has ever been
-    read. 
-    If no character can be read, sets a end of file condition and
+val peek_char : scanbuf -> char;;
+(* [Scanning.peek_char ib] returns the current char available in
+    the buffer or read one if necessary (when the current character is
+    already scanned).
+    If no character can be read, sets an end of file condition and
     returns '\000'. *)
 
 val checked_peek_char : scanbuf -> char;;
-(** Same as above but always returns a valid char or fails:
+(* Same as above but always returns a valid char or fails:
     instead of returning a null char when the reading method of the
     input buffer has reached an end of file, the function raises exception
     [End_of_file]. *)
 
 val store_char : scanbuf -> char -> int -> int;;
-(** [Scanning.store_char scanbuf c lim] adds [c] to the token buffer
+(* [Scanning.store_char ib c lim] adds [c] to the token buffer
     of the scanning buffer. It also advances the scanning buffer for one
     character and returns [lim - 1], indicating the new limit
     for the length of the current token. *)
 
-val skip_char : scanbuf -> char -> int -> int;;
-(** [Scanning.skip_char scanbuf c lim] is similar to [store_char] but
-    it ignores (does not store in the token buffer) the character [c]. *)
+val skip_char : scanbuf -> int -> int;;
+(* [Scanning.skip_char ib lim] ignores the current character. *)
+
+val ignore_char : scanbuf -> int -> int;;
+(* [Scanning.ignore_char ib lim] ignores the current character and
+   decrements the limit. *)
 
 val token : scanbuf -> string;;
-(** [Scanning.token scanbuf] returns the string stored into the token
+(* [Scanning.token ib] returns the string stored into the token
     buffer of the scanning buffer: it returns the token matched by the
     format. *)
 
 val reset_token : scanbuf -> unit;;
-(** [Scanning.reset_token scanbuf] resets the token buffer of
+(* [Scanning.reset_token ib] resets the token buffer of
     the given scanning buffer. *)
 
 val char_count : scanbuf -> int;;
-(** [Scanning.char_count scanbuf] returns the number of characters
+(* [Scanning.char_count ib] returns the number of characters
     read so far from the given buffer. *)
 
 val line_count : scanbuf -> int;;
-(** [Scanning.line_count scanbuf] returns the number of new line
+(* [Scanning.line_count ib] returns the number of new line
     characters read so far from the given buffer. *)
 
 val token_count : scanbuf -> int;;
-(** [Scanning.token_count scanbuf] returns the number of tokens read
-    so far from [scanbuf]. *)
+(* [Scanning.token_count ib] returns the number of tokens read
+    so far from [ib]. *)
 
 val eof : scanbuf -> bool;;
-(** [Scanning.eof scanbuf] returns the current value of the end of input
-    condition of the given buffer, no validity test is performed. *)
-
-val end_of_input : scanbuf -> bool;;
-(** [Scanning.end_of_input scanbuf] tests the end of input condition
+(* [Scanning.eof ib] returns the end of input condition
     of the given buffer. *)
 
+val end_of_input : scanbuf -> bool;;
+(* [Scanning.end_of_input ib] tests the end of input condition
+    of the given buffer (if no char has ever been read, an attempt to
+    read one is performed). *)
+
 val beginning_of_input : scanbuf -> bool;;
-(** [Scanning.beginning_of_input scanbuf] tests the beginning of input
+(* [Scanning.beginning_of_input ib] tests the beginning of input
     condition of the given buffer. *)
 
+val name_of_input : scanbuf -> string;;
+(* [Scanning.name_of_input ib] returns the name of the character
+    source for input buffer [ib]. *)
+
 val from_string : string -> scanbuf;;
 val from_channel : in_channel -> scanbuf;;
 val from_file : string -> scanbuf;;
@@ -105,8 +113,8 @@ type file_name = string;;
 
 type scanbuf = {
   mutable eof : bool;
-  mutable bof : bool;
-  mutable cur_char : char;
+  mutable current_char : char;
+  mutable current_char_is_valid : bool;
   mutable char_count : int;
   mutable line_count : int;
   mutable token_count : int;
@@ -115,44 +123,51 @@ type scanbuf = {
   file_name : file_name;
 };;
 
+let null_char = '\000';;
+
 (* Reads a new character from input buffer.  Next_char never fails,
    even in case of end of input: it then simply sets the end of file
    condition. *)
 let next_char ib =
   try
-   let c = ib.get_next_char () in
-   ib.cur_char <- c;
-   ib.char_count <- ib.char_count + 1;
-   if c == '\n' then ib.line_count <- ib.line_count + 1
-  with End_of_file ->
-   ib.cur_char <- '\000';
-   ib.eof <- true;;
+    let c = ib.get_next_char () in
+    ib.current_char <- c;
+    ib.current_char_is_valid <- true;
+    ib.char_count <- ib.char_count + 1;
+    if c == '\n' then ib.line_count <- ib.line_count + 1;
+    c with
+  | End_of_file ->
+    let c = null_char in
+    ib.current_char <- c;
+    ib.current_char_is_valid <- false;
+    ib.eof <- true;
+    c;;
 
-let cautious_peek_char ib =
-  if ib.bof then begin
-    next_char ib;
-    if ib.char_count > 0 then ib.bof <- false end;
-  ib.cur_char;;
+let peek_char ib =
+  if ib.current_char_is_valid then ib.current_char else next_char ib;;
 
-(* Returns a valid current char for the input buffer.  In particular
+(* Returns a valid current char for the input buffer. In particular
    no irrelevant null character (as set by [next_char] in case of end
    of input) is returned, since [End_of_file] is raised when
    [next_char] sets the end of file condition while trying to read a
    new character. *)
 let checked_peek_char ib =
-  let c = cautious_peek_char ib in
+  let c = peek_char ib in
   if ib.eof then raise End_of_file;
   c;;
 
-let peek_char ib = ib.cur_char;;
-let eof ib = ib.eof;;
-let beginning_of_input ib = ib.bof;;
 let end_of_input ib =
-  let c = cautious_peek_char ib in
+  ignore (peek_char ib);
   ib.eof;;
+
+let eof ib = ib.eof;;
+
+let beginning_of_input ib = ib.char_count = 0;;
+let name_of_input ib = ib.file_name;;
 let char_count ib = ib.char_count;;
 let line_count ib = ib.line_count;;
 let reset_token ib = Buffer.reset ib.tokbuf;;
+let invalidate_current_char ib = ib.current_char_is_valid <- false;;
 
 let token ib =
   let tokbuf = ib.tokbuf in
@@ -163,21 +178,22 @@ let token ib =
 
 let token_count ib = ib.token_count;;
 
+let skip_char ib max =
+  invalidate_current_char ib;
+  max;;
+
+let ignore_char ib max = skip_char ib (max - 1);;
+
 let store_char ib c max =
   Buffer.add_char ib.tokbuf c;
-  next_char ib;
-  max - 1;;
-
-let skip_char ib c max =
-  next_char ib;
-  max - 1;;
+  ignore_char ib max;;
 
 let default_token_buffer_size = 1024;;
 
 let create fname next = {
   eof = false;
-  bof = true;
-  cur_char = '\000';
+  current_char = '\000';
+  current_char_is_valid = false;
   char_count = 0;
   line_count = 0;
   token_count = 0;
@@ -194,9 +210,9 @@ let from_string s =
     let c = s.[!i] in
     incr i;
     c in
-  create "string" next;;
+  create "string input" next;;
 
-let from_function = create "function";;
+let from_function = create "function input";;
 
 (* Perform bufferized input to improve efficiency. *)
 let file_buffer_size = ref 1024;;
@@ -223,14 +239,14 @@ let from_input_channel fname ic =
   let next () = input_char ic in
   create fname next;;
 
-let from_channel = from_input_channel "in_channel";;
+let from_channel = from_input_channel "input channel";;
 
+(* The scanning buffer reading from [stdin].*)
 let stdib = from_input_channel "stdin" stdin;;
-(** The scanning buffer reading from [stdin].*)
 
 end;;
 
-(** Formatted input functions. *)
+(* Formatted input functions. *)
 
 (* Reporting errors. *)
 exception Scan_failure of string;;
@@ -244,31 +260,54 @@ let bad_input_escape c =
 let scanf_bad_input ib = function
   | Scan_failure s | Failure s ->
       let i = Scanning.char_count ib in
-      bad_input (Printf.sprintf "scanf: bad input at char number %i: %S" i s)
+      bad_input (Printf.sprintf "scanf: bad input at char number %i: %s" i s)
   | x -> raise x;;
 
-let bad_format fmt i fc =
+let bad_conversion fmt i c =
   invalid_arg
     (Printf.sprintf
-       "scanf: bad conversion %%%c, at char number %i in format %S" fc i fmt);;
+       "scanf: bad conversion %%%c, at char number %i \
+        in format string ``%s''" c i fmt);;
+
+let incomplete_format fmt =
+  invalid_arg
+    (Printf.sprintf "scanf: premature end of format string ``%s''" fmt);;
 
 let bad_float () = bad_input "no dot or exponent part found in float token";;
 
-(* Checking that the current char is indeed one of range, then skip it. *)
-let check_char_in range ib =
-  if range <> [] && not (Scanning.end_of_input ib) then
-  let ci = Scanning.checked_peek_char ib in
-  if List.memq ci range then Scanning.next_char ib else
-  let sr = String.concat "" (List.map (String.make 1) range) in
-  bad_input
-    (Printf.sprintf "looking for one of range %S, found %C" sr ci);;
+let format_mismatch_err fmt1 fmt2 =
+  Printf.sprintf "format read %S does not match specification %S" fmt1 fmt2;;
 
-(* Checking that [c] is indeed in the input, then skip it. *)
+let format_mismatch fmt1 fmt2 ib =
+  scanf_bad_input ib (Scan_failure (format_mismatch_err fmt1 fmt2));;
+
+(* Checking that 2 format string are type compatible. *)
+let compatible_format_type fmt1 fmt2 =
+  Printf.summarize_format_type fmt1 = Printf.summarize_format_type fmt2;;
+
+(* Checking that [c] is indeed in the input, then skips it.
+   In this case, the character c has been explicitely specified in the
+   format as being mandatory in the input; hence we should fail with
+   End_of_file in case of end_of_input.
+   That's why we use checked_peek_char here. *)
 let check_char ib c =
   let ci = Scanning.checked_peek_char ib in
-  if ci != c
-  then bad_input (Printf.sprintf "looking for %C, found %C" c ci)
-  else Scanning.next_char ib;;
+  if ci != c then
+    bad_input (Printf.sprintf "looking for %C, found %C" c ci) else
+  Scanning.invalidate_current_char ib;;
+
+(* Checks that the current char is indeed one of the stopper characters,
+   then skips it.
+   Be careful that if ib has no more character this procedure should
+   just do nothing (since %s@c defaults to the entire rest of the
+   buffer, when no character c can be found in the input). *)
+let ignore_stoppers stps ib =
+  if stps <> [] && not (Scanning.eof ib) then
+  let ci = Scanning.peek_char ib in
+  if List.memq ci stps then Scanning.invalidate_current_char ib else
+  let sr = String.concat "" (List.map (String.make 1) stps) in
+  bad_input
+    (Printf.sprintf "looking for one of range %S, found %C" sr ci);;
 
 (* Extracting tokens from ouput token buffer. *)
 
@@ -328,69 +367,69 @@ let token_int64 conv ib = int64_of_string (token_int_literal conv ib);;
    scanning function). *)
 
 (* The decimal case is treated especially for optimization purposes. *)
-let scan_decimal_digits max ib =
-  let rec loop inside max =
-    if max = 0 || Scanning.eof ib then max else
-    match Scanning.cautious_peek_char ib with
-    | '0' .. '9' as c ->
-        let max = Scanning.store_char ib c max in
-        loop true max
-    | '_' as c when inside ->
-       let max = Scanning.skip_char ib c max in
-       loop true max
-    | c -> max in
-  loop false max;;
+let rec scan_decimal_digits max ib =
+  if max = 0 then max else
+  let c = Scanning.peek_char ib in
+  if Scanning.eof ib then max else
+  match c with
+  | '0' .. '9' as c ->
+     let max = Scanning.store_char ib c max in
+     scan_decimal_digits max ib
+  | '_' ->
+     let max = Scanning.ignore_char ib max in
+     scan_decimal_digits max ib
+  | _ -> max;;
 
-(* To scan numbers from other bases, we use a predicate argument to
-   scan_digits. *)
-let scan_digits digitp max ib =
-  let rec loop inside max =
-    if max = 0 || Scanning.eof ib then max else
-    match Scanning.cautious_peek_char ib with
-    | c when digitp c ->
-       let max = Scanning.store_char ib c max in
-       loop true max
-    | '_' as c when inside ->
-       let max = Scanning.skip_char ib c max in
-       loop true max
-    | _ -> max in
-  loop false max;;
+let scan_decimal_digits_plus max ib =
+  let c = Scanning.checked_peek_char ib in
+  match c with
+  | '0' .. '9' ->
+    let max = Scanning.store_char ib c max in
+    scan_decimal_digits max ib
+  | c -> bad_input_char c;;
 
 let scan_digits_plus digitp max ib =
+  (* To scan numbers from other bases, we use a predicate argument to
+     scan_digits. *)
+  let rec scan_digits max =
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
+    match c with
+    | c when digitp c ->
+       let max = Scanning.store_char ib c max in
+       scan_digits max
+    | '_' ->
+       let max = Scanning.ignore_char ib max in
+       scan_digits max
+    | _ -> max in
+
   let c = Scanning.checked_peek_char ib in
   if digitp c then
     let max = Scanning.store_char ib c max in
-    scan_digits digitp max ib
+    scan_digits max
   else bad_input_char c;;
 
 let is_binary_digit = function
   | '0' .. '1' -> true
   | _ -> false;;
 
-let scan_binary_digits = scan_digits is_binary_digit;;
 let scan_binary_int = scan_digits_plus is_binary_digit;;
 
 let is_octal_digit = function
   | '0' .. '7' -> true
   | _ -> false;;
 
-let scan_octal_digits = scan_digits is_octal_digit;;
 let scan_octal_int = scan_digits_plus is_octal_digit;;
 
 let is_hexa_digit = function
   | '0' .. '9' | 'a' .. 'f' | 'A' .. 'F' -> true
   | _ -> false;;
 
-let scan_hexadecimal_digits = scan_digits is_hexa_digit;;
 let scan_hexadecimal_int = scan_digits_plus is_hexa_digit;;
 
 (* Scan a decimal integer. *)
-let scan_unsigned_decimal_int max ib =
-  match Scanning.checked_peek_char ib with
-  | '0' .. '9' as c ->
-      let max = Scanning.store_char ib c max in
-      scan_decimal_digits max ib
-  | c -> bad_input_char c;;
+let scan_unsigned_decimal_int = scan_decimal_digits_plus;;
 
 let scan_sign max ib =
   let c = Scanning.checked_peek_char ib in
@@ -411,12 +450,13 @@ let scan_unsigned_int max ib =
   match Scanning.checked_peek_char ib with
   | '0' as c ->
       let max = Scanning.store_char ib c max in
-      if max = 0 || Scanning.eof ib then max else
+      if max = 0 then max else
       let c = Scanning.peek_char ib in
+      if Scanning.eof ib then max else
       begin match c with
-      | 'x' | 'X' -> scan_hexadecimal_digits (Scanning.store_char ib c max) ib
-      | 'o' -> scan_octal_digits (Scanning.store_char ib c max) ib
-      | 'b' -> scan_binary_digits (Scanning.store_char ib c max) ib
+      | 'x' | 'X' -> scan_hexadecimal_int (Scanning.store_char ib c max) ib
+      | 'o' -> scan_octal_int (Scanning.store_char ib c max) ib
+      | 'b' -> scan_binary_int (Scanning.store_char ib c max) ib
       | c -> scan_decimal_digits max ib end
   | c -> scan_unsigned_decimal_int max ib;;
 
@@ -437,31 +477,37 @@ let scan_int_conv conv max ib =
 (* Scanning floating point numbers. *)
 (* Fractional part is optional and can be reduced to 0 digits. *)
 let scan_frac_part max ib =
-  if max = 0 || Scanning.eof ib then max else
-  scan_decimal_digits max ib;;
+  if max = 0 then max else
+  let c = Scanning.peek_char ib in
+  if Scanning.eof ib then max else
+  match c with
+  | '0' .. '9' as c ->
+    scan_decimal_digits (Scanning.store_char ib c max) ib
+  | _ -> max;;
 
 (* Exp part is optional and can be reduced to 0 digits. *)
 let scan_exp_part max ib =
-  if max = 0 || Scanning.eof ib then max else
+  if max = 0 then max else
   let c = Scanning.peek_char ib in
+  if Scanning.eof ib then max else
   match c with
   | 'e' | 'E' as c ->
      scan_optionally_signed_decimal_int (Scanning.store_char ib c max) ib
   | _ -> max;;
 
-(* An optional sign followed by a possibly empty sequence of decimal digits. *)
-let scan_optionally_signed_decimal_digits max ib =
+(* Scan the integer part of a floating point number, (not using the
+   Caml lexical convention since the integer part can be empty):
+   an optional sign, followed by a possibly empty sequence of decimal
+   digits (e.g. -.1). *)
+let scan_int_part max ib =
   let max = scan_sign max ib in
   scan_decimal_digits max ib;;
 
-(* Scan the integer part of a floating point number, (not using the
-   Caml lexical convention since the integer part can be empty). *)
-let scan_int_part = scan_optionally_signed_decimal_digits;;
-
 let scan_float max ib =
   let max = scan_int_part max ib in
-  if max = 0 || Scanning.eof ib then max else
+  if max = 0 then max else
   let c = Scanning.peek_char ib in
+  if Scanning.eof ib then max else
   match c with
   | '.' ->
      let max = Scanning.store_char ib c max in
@@ -471,8 +517,9 @@ let scan_float max ib =
 
 let scan_Float max ib =
   let max = scan_optionally_signed_decimal_int max ib in
-  if max = 0 || Scanning.eof ib then bad_float () else
+  if max = 0 then bad_float () else
   let c = Scanning.peek_char ib in
+  if Scanning.eof ib then bad_float () else
   match c with
   | '.' ->
      let max = Scanning.store_char ib c max in
@@ -487,17 +534,16 @@ let scan_Float max ib =
    characters has been read.*)
 let scan_string stp max ib =
   let rec loop max =
-    if max = 0 || Scanning.end_of_input ib then max else
-    let c = Scanning.checked_peek_char ib in
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if stp == [] then
       match c with
       | ' ' | '\t' | '\n' | '\r' -> max
       | c -> loop (Scanning.store_char ib c max) else
-    if List.mem c stp then max else
+    if List.memq c stp then Scanning.skip_char ib max else
     loop (Scanning.store_char ib c max) in
-  let max = loop max in
-  check_char_in stp ib;
-  max;;
+  loop max;;
 
 (* Scan a char: peek strictly one character in the input, whatsoever. *)
 let scan_char max ib =
@@ -526,15 +572,15 @@ let char_for_decimal_code c0 c1 c2 =
 (* Called when encountering '\\' as starter of a char.
    Stops before the corresponding '\''. *)
 let scan_backslash_char max ib =
-  if max = 0 || Scanning.eof ib then bad_input "a char" else
+  if max = 0 then bad_input "a char" else
   let c = Scanning.peek_char ib in
+  if Scanning.eof ib then bad_input "a char" else
   match c with
   | '\\' | '\'' | '"' | 'n' | 't' | 'b' | 'r' (* '"' helping Emacs *) ->
      Scanning.store_char ib (char_for_backslash c) max
   | '0' .. '9' as c ->
      let get_digit () =
-       Scanning.next_char ib;
-       let c = Scanning.peek_char ib in
+       let c = Scanning.next_char ib in
        match c with
        | '0' .. '9' as c -> c
        | c -> bad_input_escape c in
@@ -546,49 +592,53 @@ let scan_backslash_char max ib =
 
 let scan_Char max ib =
   let rec loop s max =
-   if max = 0 || Scanning.eof ib then bad_input "a char" else
+   if max = 0 then bad_input "a char" else
    let c = Scanning.checked_peek_char ib in
+   if Scanning.eof ib then bad_input "a char" else
    match c, s with
-   | '\'', 3 -> Scanning.next_char ib; loop 2 (max - 1)
-   | '\'', 1 -> Scanning.next_char ib; max - 1
-   | '\\', 2 -> Scanning.next_char ib;
-                loop 1 (scan_backslash_char (max - 1) ib)
+   | '\'', 3 -> loop 2 (Scanning.ignore_char ib max)
+   | '\'', 1 -> Scanning.ignore_char ib max
+   | '\\', 2 -> loop 1 (scan_backslash_char (Scanning.ignore_char ib max) ib)
    | c, 2 -> loop 1 (Scanning.store_char ib c max)
    | c, _ -> bad_input_escape c in
   loop 3 max;;
 
 let scan_String max ib =
   let rec loop s max =
-    if max = 0 || Scanning.eof ib then bad_input "a string" else
+    if max = 0 then bad_input "a string" else
     let c = Scanning.checked_peek_char ib in
+    if Scanning.eof ib then bad_input "a string" else
     match c, s with
     | '"', true (* '"' helping Emacs *) ->
-       Scanning.next_char ib; loop false (max - 1)
+       loop false (Scanning.ignore_char ib max)
     | '"', false (* '"' helping Emacs *) ->
-       Scanning.next_char ib; max - 1
+       Scanning.ignore_char ib max
     | '\\', false ->
-       Scanning.next_char ib; skip_spaces true (max - 1)
+       skip_spaces true (Scanning.ignore_char ib max)
     | c, false -> loop false (Scanning.store_char ib c max)
     | c, _ -> bad_input_char c
   and skip_spaces s max =
-    if max = 0 || Scanning.eof ib then bad_input "a string" else
+    if max = 0 then bad_input "a string" else
     let c = Scanning.checked_peek_char ib in
+    if Scanning.eof ib then bad_input "a string" else
     match c, s with
     | '\n', true
     | ' ', false ->
-       Scanning.next_char ib; skip_spaces false (max - 1)
+       skip_spaces false (Scanning.ignore_char ib max)
     | '\\', false -> loop false max
     | c, false -> loop false (Scanning.store_char ib c max)
     | _, _ -> loop false (scan_backslash_char (max - 1) ib) in
   loop true max;;
 
 let scan_bool max ib =
-  if max < 4 || Scanning.eof ib then bad_input "a boolean" else
+  if max < 4 then bad_input "a boolean" else
+  let c = Scanning.checked_peek_char ib in
+  if Scanning.eof ib then bad_input "a boolean" else
   let m =
-    match Scanning.checked_peek_char ib with
+    match c with
     | 't' -> 4
     | 'f' -> 5
-    | _ -> 0 in
+    | _ -> bad_input "a boolean" in
   scan_string [] (min max m) ib;;
 
 (* Reading char sets in %[...] conversions. *)
@@ -601,18 +651,18 @@ let read_char_set fmt i =
   let lim = String.length fmt - 1 in
 
   let rec find_in_set j =
-    if j > lim then bad_format fmt j fmt.[lim - 1] else
+    if j > lim then incomplete_format fmt else
     match fmt.[j] with
     | ']' -> j
     | c -> find_in_set (j + 1)
 
   and find_set i =
-    if i > lim then bad_format fmt i fmt.[lim - 1] else
+    if i > lim then incomplete_format fmt else
     match fmt.[i] with
     | ']' -> find_in_set (i + 1)
     | c -> find_in_set i in
 
-  if i > lim then bad_format fmt i fmt.[lim - 1] else
+  if i > lim then incomplete_format fmt else
   match fmt.[i] with
   | '^' ->
      let i = i + 1 in
@@ -656,16 +706,18 @@ let get_char_in_range r c = get_bit_of_range r (int_of_char c);;
 
 let bit_not b = (lnot b) land 1;;
 
-(* Build the bit vector corresponding to a char set read in the format. *)
-let make_bv bit set =
+(* Build the bit vector corresponding to the set of characters
+   that belongs to the string argument [set].
+   (In the Scanf module [set] is always a sub-string of the format). *)
+let make_char_bit_vect bit set =
   let r = make_range (bit_not bit) in
   let lim = String.length set - 1 in
   let rec loop bit rp i =
     if i <= lim then
     match set.[i] with
     | '-' when rp ->
-       (* if i = 0 then rp is false (since the initial call is loop bit false 0)
-          hence i >= 1 and the following is safe. *)
+       (* if i = 0 then rp is false (since the initial call is
+          loop bit false 0). Hence i >= 1 and the following is safe. *)
        let c1 = set.[i - 1] in
        let i = i + 1 in
        if i > lim then loop bit false (i - 1) else
@@ -681,7 +733,7 @@ let make_bv bit set =
 
 (* Compute the predicate on chars corresponding to a char set. *)
 let make_pred bit set stp =
-  let r = make_bv bit set in
+  let r = make_char_bit_vect bit set in
   List.iter
     (fun c -> set_bit_of_range r (int_of_char c) (bit_not bit)) stp;
   (fun c -> get_char_in_range r c);;
@@ -739,46 +791,54 @@ let find_setp stp char_set =
 
 let scan_chars_in_char_set stp char_set max ib =
   let rec loop_pos1 cp1 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c == cp1
     then loop_pos1 cp1 (Scanning.store_char ib c max)
     else max
   and loop_pos2 cp1 cp2 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c == cp1 || c == cp2
     then loop_pos2 cp1 cp2 (Scanning.store_char ib c max)
     else max
   and loop_pos3 cp1 cp2 cp3 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c == cp1 || c == cp2 || c == cp3
     then loop_pos3 cp1 cp2 cp3 (Scanning.store_char ib c max)
     else max
   and loop_neg1 cp1 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c != cp1
     then loop_neg1 cp1 (Scanning.store_char ib c max)
     else max
   and loop_neg2 cp1 cp2 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c != cp1 && c != cp2
     then loop_neg2 cp1 cp2 (Scanning.store_char ib c max)
     else max
   and loop_neg3 cp1 cp2 cp3 max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
     if c != cp1 && c != cp2 && c != cp3
     then loop_neg3 cp1 cp2 cp3 (Scanning.store_char ib c max)
     else max
   and loop setp max =
-    let c = Scanning.cautious_peek_char ib in
-    if max = 0 || Scanning.end_of_input ib then max else
-    if setp c == 1 then loop setp (Scanning.store_char ib c max) else
-    max in
+    if max = 0 then max else
+    let c = Scanning.peek_char ib in
+    if Scanning.eof ib then max else
+    if setp c == 1
+    then loop setp (Scanning.store_char ib c max)
+    else max in
 
   let max =
     match char_set with
@@ -796,7 +856,7 @@ let scan_chars_in_char_set stp char_set max ib =
         | 2 -> loop_neg2 set.[0] set.[1] max
         | 3 when set.[1] != '-' -> loop_neg3 set.[0] set.[1] set.[2] max
         | n -> loop (find_setp stp char_set) max end in
-  check_char_in stp ib;
+  ignore_stoppers stp ib;
   max;;
 
 let get_count t ib =
@@ -805,14 +865,19 @@ let get_count t ib =
   | 'n' -> Scanning.char_count ib
   | _ -> Scanning.token_count ib;;
 
-let skip_whites ib =
-  let rec loop = function
-  | ' ' | '\t' | '\n' | '\r' ->
-     Scanning.next_char ib;
-     if not (Scanning.eof ib) then loop (Scanning.peek_char ib)
-  | _ -> () in
-  if not (Scanning.eof ib) then
-  loop (Scanning.cautious_peek_char ib);;
+let rec skip_whites ib =
+  let c = Scanning.peek_char ib in
+  if not (Scanning.eof ib) then begin
+    match c with
+    | ' ' | '\t' | '\n' | '\r' ->
+      Scanning.invalidate_current_char ib; skip_whites ib
+    | _ -> ()
+  end;;
+
+external format_to_string :
+ ('a, 'b, 'c, 'd) format4 -> string = "%identity";;
+external string_to_format :
+ string -> ('a, 'b, 'c, 'd) format4 = "%identity";;
 
 (* The [kscanf] main scanning function.
    It takes as arguments:
@@ -834,7 +899,7 @@ let skip_whites ib =
    aborts and applies the scanning buffer and a string that explains
    the error to the error handling function [ef] (the error continuation). *)
 let kscanf ib ef fmt f =
-  let fmt = string_of_format fmt in
+  let fmt = format_to_string fmt in
   let lim = String.length fmt - 1 in
 
   let return v = Obj.magic v () in
@@ -847,11 +912,11 @@ let kscanf ib ef fmt f =
     match fmt.[i] with
     | ' ' -> skip_whites ib; scan_fmt f (i + 1)
     | '%' ->
-        if i > lim then bad_format fmt i '%' else
+        if i > lim then incomplete_format fmt else
         scan_conversion false max_int f (i + 1)
-    | '@' as t ->
+    | '@' ->
         let i = i + 1 in
-        if i > lim then bad_format fmt (i - 1) t else begin
+        if i > lim then incomplete_format fmt else begin
         check_char ib fmt.[i];
         scan_fmt f (i + 1) end
     | c -> check_char ib c; scan_fmt f (i + 1)
@@ -859,60 +924,60 @@ let kscanf ib ef fmt f =
   and scan_conversion skip max f i =
     let stack = if skip then no_stack else stack in
     match fmt.[i] with
-    | '%' as c ->
-        check_char ib c; scan_fmt f (i + 1)
+    | '%' as conv ->
+        check_char ib conv; scan_fmt f (i + 1)
     | 'c' when max = 0 ->
         let c = Scanning.checked_peek_char ib in
         scan_fmt (stack f c) (i + 1)
     | 'c' | 'C' as conv ->
-        if max <> 1 && max <> max_int then bad_format fmt i conv else
-        let x =
+        if max <> 1 && max <> max_int then bad_conversion fmt i conv else
+        let _x =
           if conv = 'c' then scan_char max ib else scan_Char max ib in
         scan_fmt (stack f (token_char ib)) (i + 1)
     | 'd' | 'i' | 'o' | 'u' | 'x' | 'X' as conv ->
-        let x = scan_int_conv conv max ib in
+        let _x = scan_int_conv conv max ib in
         scan_fmt (stack f (token_int conv ib)) (i + 1)
     | 'f' | 'g' | 'G' | 'e' | 'E' ->
-        let x = scan_float max ib in
+        let _x = scan_float max ib in
         scan_fmt (stack f (token_float ib)) (i + 1)
     | 'F' ->
-        let x = scan_Float max ib in
+        let _x = scan_Float max ib in
         scan_fmt (stack f (token_float ib)) (i + 1)
     | 's' ->
         let i, stp = scan_fmt_stoppers (i + 1) in
-        let x = scan_string stp max ib in
+        let _x = scan_string stp max ib in
         scan_fmt (stack f (token_string ib)) (i + 1)
     | '[' ->
         let i, char_set = read_char_set fmt (i + 1) in
         let i, stp = scan_fmt_stoppers (i + 1) in
-        let x = scan_chars_in_char_set stp char_set max ib in
+        let _x = scan_chars_in_char_set stp char_set max ib in
         scan_fmt (stack f (token_string ib)) (i + 1)
     | 'S' ->
-        let x = scan_String max ib in
+        let _x = scan_String max ib in
         scan_fmt (stack f (token_string ib)) (i + 1)
     | 'B' | 'b' ->
-        let x = scan_bool max ib in
+        let _x = scan_bool max ib in
         scan_fmt (stack f (token_bool ib)) (i + 1)
-    | 'l' | 'n' | 'L' as t ->
+    | 'l' | 'n' | 'L' as conv ->
         let i = i + 1 in
-        if i > lim then scan_fmt (stack f (get_count t ib)) i else begin
+        if i > lim then scan_fmt (stack f (get_count conv ib)) i else begin
         match fmt.[i] with
         | 'd' | 'i' | 'o' | 'u' | 'x' | 'X' as conv ->
-            let x = scan_int_conv conv max ib in
-            begin match t with
+            let _x = scan_int_conv conv max ib in
+            begin match conv with
             | 'l' -> scan_fmt (stack f (token_int32 conv ib)) (i + 1)
-            | 'L' -> scan_fmt (stack f (token_int64 conv ib)) (i + 1)
-            | _ -> scan_fmt (stack f (token_nativeint conv ib)) (i + 1) end
-        | c -> scan_fmt (stack f (get_count t ib)) i end
-    | 'N' as t ->
-        scan_fmt (stack f (get_count t ib)) (i + 1)
-    | '!' as c ->
+            | 'n' -> scan_fmt (stack f (token_nativeint conv ib)) (i + 1)
+            | _ -> scan_fmt (stack f (token_int64 conv ib)) (i + 1) end
+        | c -> scan_fmt (stack f (get_count conv ib)) i end
+    | 'N' as conv ->
+        scan_fmt (stack f (get_count conv ib)) (i + 1)
+    | '!' ->
         if Scanning.end_of_input ib then scan_fmt f (i + 1)
         else bad_input "end of input not found"
     | '_' ->
-        if i > lim then bad_format fmt i fmt.[lim - 1] else
+        if i > lim then incomplete_format fmt else
         scan_conversion true max f (i + 1)
-    | '0' .. '9' as c ->
+    | '0' .. '9' as conv ->
         let rec read_width accu i =
           if i > lim then accu, i else
           match fmt.[i] with
@@ -920,16 +985,32 @@ let kscanf ib ef fmt f =
              let accu = 10 * accu + int_value_of_char c in
              read_width accu (i + 1)
           | _ -> accu, i in
-        let max, i = read_width (int_value_of_char c) (i + 1) in
-        if i > lim then bad_format fmt i fmt.[lim - 1] else
-        scan_conversion skip max f i
-    | c -> bad_format fmt i c
+        let max, i = read_width (int_value_of_char conv) (i + 1) in
+        if i > lim then incomplete_format fmt else begin
+        match fmt.[i] with
+        | '.' ->
+          let p, i = read_width 0 (i + 1) in
+          scan_conversion skip (max + p + 1) f i
+        | _ -> scan_conversion skip max f i end
+    | '(' | '{' as conv ->
+        let i = succ i in
+        let j =
+          Printf.sub_format incomplete_format bad_conversion conv fmt i + 1 in
+        let mf = String.sub fmt i (j - i - 2) in
+        let _x = scan_String max ib in
+        let rf = token_string ib in
+        if not (compatible_format_type mf rf)
+          then format_mismatch rf mf ib else 
+        if conv = '{' then scan_fmt (stack f rf) j else
+        let nf = scan_fmt (Obj.magic rf) 0 in
+        scan_fmt (stack f nf) j
+    | c -> bad_conversion fmt i c
 
   and scan_fmt_stoppers i =
     if i > lim then i - 1, [] else
     match fmt.[i] with
     | '@' when i < lim -> let i = i + 1 in i, [fmt.[i]]
-    | '@' as c when i = lim -> bad_format fmt i c
+    | '@' when i = lim -> incomplete_format fmt
     | _ -> i - 1, [] in
 
   Scanning.reset_token ib;
@@ -947,3 +1028,16 @@ let fscanf ic = bscanf (Scanning.from_channel ic);;
 let sscanf s = bscanf (Scanning.from_string s);;
 
 let scanf fmt = bscanf Scanning.stdib fmt;;
+
+let bscanf_format ib fmt2 f =
+  let fmt1 = ignore (scan_String max_int ib); token_string ib in
+  let fmt2 = format_to_string fmt2 in
+  if compatible_format_type fmt1 fmt2
+  then let fresh_fmt = String.copy fmt1 in f (string_to_format fresh_fmt)
+  else format_mismatch fmt1 fmt2 ib;;
+
+let sscanf_format s fmt =
+  let fmt = format_to_string fmt in
+  if compatible_format_type s fmt
+  then let fresh_fmt = String.copy s in string_to_format fresh_fmt
+  else bad_input (format_mismatch_err s fmt);;
