@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: toploop.ml,v 1.87 2004/06/12 08:55:47 xleroy Exp $ *)
+(* $Id: toploop.ml,v 1.92 2005/08/08 09:41:51 xleroy Exp $ *)
 
 (* The interactive toplevel loop *)
 
@@ -148,7 +148,7 @@ let load_lambda ppf lam =
 
 (* Print the outcome of an evaluation *)
 
-let pr_item env = function
+let rec pr_item env = function
   | Tsig_value(id, decl) :: rem ->
       let tree = Printtyp.tree_of_value_description id decl in
       let valopt =
@@ -162,6 +162,8 @@ let pr_item env = function
             Some v
       in
       Some (tree, valopt, rem)
+  | Tsig_type(id, _, _) :: rem when Btype.is_row_name (Ident.name id) ->
+      pr_item env rem
   | Tsig_type(id, decl, rs) :: rem ->
       let tree = Printtyp.tree_of_type_declaration id decl rs in
       Some (tree, None, rem)
@@ -339,7 +341,8 @@ let read_interactive_input = ref read_input_default
 let refill_lexbuf buffer len =
   if !got_eof then (got_eof := false; 0) else begin
     let prompt =
-      if !first_line then "# "
+      if !Clflags.noprompt then ""
+      else if !first_line then "# "
       else if Lexer.in_comment () then "* "
       else "  "
     in
@@ -367,11 +370,16 @@ let _ =
     crc_intfs
 
 let load_ocamlinit ppf =
-  let home_init = 
-    try Filename.concat (Sys.getenv "HOME") ".ocamlinit"
-    with Not_found -> ".ocamlinit" in
-  if Sys.file_exists ".ocamlinit" then ignore(use_silently ppf ".ocamlinit")
-  else if Sys.file_exists home_init then ignore(use_silently ppf home_init)
+  match !Clflags.init_file with
+  | Some f -> if Sys.file_exists f then ignore (use_silently ppf f)
+              else fprintf ppf "Init file not found: \"%s\".@." f
+  | None ->
+     if Sys.file_exists ".ocamlinit" then ignore (use_silently ppf ".ocamlinit")
+     else try
+       let home_init = Filename.concat (Sys.getenv "HOME") ".ocamlinit" in
+       if Sys.file_exists home_init then ignore (use_silently ppf home_init)
+     with Not_found -> ()
+;;
 
 let set_paths () =
   (* Add whatever -I options have been specified on the command line,
