@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: printtyp.ml,v 1.130.2.3 2005/04/15 08:16:41 garrigue Exp $ *)
+(* $Id: printtyp.ml,v 1.139 2005/08/16 00:48:56 garrigue Exp $ *)
 
 (* Printing functions *)
 
@@ -207,7 +207,7 @@ let aliased = ref ([] : type_expr list)
 let delayed = ref ([] : type_expr list)
 
 let add_delayed t =
-  if not (List.mem_assq t !names) then delayed := t :: !delayed
+  if not (List.memq t !delayed) then delayed := t :: !delayed
 
 let is_aliased ty = List.memq (proxy ty) !aliased
 let add_alias ty =
@@ -377,9 +377,11 @@ let rec tree_of_typexp sch ty =
         let tyl = List.map repr tyl in
         (* let tyl = List.filter is_aliased tyl in *)
         if tyl = [] then tree_of_typexp sch ty else begin
+          let old_delayed = !delayed in
           List.iter add_delayed tyl;
           let tl = List.map name_of_type tyl in
-          Otyp_poly (tl, tree_of_typexp sch ty)
+          let tr = Otyp_poly (tl, tree_of_typexp sch ty) in
+          delayed := old_delayed; tr
         end
     | Tunivar ->
         Otyp_var (false, name_of_type ty)
@@ -436,6 +438,7 @@ and tree_of_typfields sch rest = function
       let rest =
         match rest.desc with
         | Tvar | Tunivar -> Some (is_non_gen sch rest)
+        | Tconstr _ -> Some false
         | Tnil -> None
         | _ -> fatal_error "typfields (1)"
       in
@@ -530,26 +533,25 @@ let rec tree_of_type_decl id decl =
     | _ -> "?"
   in
   let type_defined decl =
-    if List.exists2
-        (fun ty x -> x <> (true,true,true) &&
-          (decl.type_kind = Type_abstract && ty_manifest = None
-         || (repr ty).desc <> Tvar))
+    let abstr =
+      match decl.type_kind with
+        Type_abstract ->
+          begin match decl.type_manifest with
+            None -> true
+          | Some ty -> has_constr_row ty
+          end
+      | Type_variant(_,p) | Type_record(_,_,p) ->
+          p = Private
+    in
+    let vari =
+      List.map2
+        (fun ty (co,cn,ct) ->
+          if abstr || (repr ty).desc <> Tvar then (co,cn) else (true,true))
         decl.type_params decl.type_variance
-    then
-      let vari = List.map (fun (co,cn,ct) -> (co,cn)) decl.type_variance in
-      (Ident.name id,
-       List.combine
-         (List.map (fun ty -> type_param (tree_of_typexp false ty)) params)
-         vari)
-    else
-      let ty =
-        tree_of_typexp false
-          (Btype.newgenty (Tconstr(Pident id, params, ref Mnil)))
-      in
-      match ty with
-      | Otyp_constr (Oide_ident id, tyl) ->
-          (id, List.map (fun ty -> (type_param ty, (true, true))) tyl)
-      | _ -> ("?", [])
+    in
+    (Ident.name id,
+     List.map2 (fun ty cocn -> type_param (tree_of_typexp false ty), cocn)
+       params vari)
   in
   let tree_of_manifest ty1 =
     match ty_manifest with
@@ -558,19 +560,21 @@ let rec tree_of_type_decl id decl =
   in
   let (name, args) = type_defined decl in
   let constraints = tree_of_constraints params in
-  let ty =
+  let ty, priv =
     match decl.type_kind with
     | Type_abstract ->
         begin match ty_manifest with
-        | None -> Otyp_abstract
-        | Some ty -> tree_of_typexp false ty
+        | None -> (Otyp_abstract, Public)
+        | Some ty ->
+            tree_of_typexp false ty, 
+            (if has_constr_row ty then Private else Public)
         end
     | Type_variant(cstrs, priv) ->
-        tree_of_manifest (Otyp_sum (List.map tree_of_constructor cstrs, priv))
+        tree_of_manifest (Otyp_sum (List.map tree_of_constructor cstrs)), priv
     | Type_record(lbls, rep, priv) ->
-        tree_of_manifest (Otyp_record (List.map tree_of_label lbls, priv))
+        tree_of_manifest (Otyp_record (List.map tree_of_label lbls)), priv
   in
-  (name, args, ty, constraints)
+  (name, args, ty, priv, constraints)
 
 and tree_of_constructor (name, args) =
   (name, tree_of_typlist false args)
@@ -680,6 +684,8 @@ let rec tree_of_class_type sch params =
       in
       let all_vars =
         Vars.fold (fun l (m, t) all -> (l, m, t) :: all) sign.cty_vars [] in
+      (* Consequence of PR#3607: order of Map.fold has changed! *)
+      let all_vars = List.rev all_vars in
       let csil =
         List.fold_left
           (fun csil (l, m, t) ->
@@ -706,6 +712,12 @@ let class_type ppf cty =
   prepare_class_type [] cty;
   !Oprint.out_class_type ppf (tree_of_class_type false [] cty)
 
+let tree_of_class_param param variance =
+  (match tree_of_typexp true param with
+    Otyp_var (_, s) -> s
+  | _ -> "?"),
+  if (repr param).desc = Tvar then (true, true) else variance
+
 let tree_of_class_params params =
   let tyl = tree_of_typlist true params in
   List.map (function Otyp_var (_, s) -> s | _ -> "?") tyl
@@ -724,7 +736,8 @@ let tree_of_class_declaration id cl rs =
 
   let vir_flag = cl.cty_new = None in
   Osig_class
-    (vir_flag, Ident.name id, tree_of_class_params params,
+    (vir_flag, Ident.name id,
+     List.map2 tree_of_class_param params cl.cty_variance,
      tree_of_class_type true params cl.cty_type,
      tree_of_rec rs)
 
@@ -754,7 +767,8 @@ let tree_of_cltype_declaration id cl rs =
       fields in
 
   Osig_class_type
-    (virt, Ident.name id, tree_of_class_params params,
+    (virt, Ident.name id,
+     List.map2 tree_of_class_param params cl.clty_variance,
      tree_of_class_type true params cl.clty_type,
      tree_of_rec rs)
 
@@ -776,6 +790,8 @@ and tree_of_signature = function
   | [] -> []
   | Tsig_value(id, decl) :: rem ->
       tree_of_value_description id decl :: tree_of_signature rem
+  | Tsig_type(id, _, _) :: rem when is_row_name (Ident.name id) ->
+      tree_of_signature rem
   | Tsig_type(id, decl, rs) :: rem ->
       Osig_type(tree_of_type_decl id decl, tree_of_rec rs) ::
       tree_of_signature rem
