@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: subst.ml,v 1.46 2004/06/12 08:55:48 xleroy Exp $ *)
+(* $Id: subst.ml,v 1.48 2005/03/23 03:08:37 garrigue Exp $ *)
 
 (* Substitutions *)
 
@@ -106,14 +106,18 @@ let rec typexp s ty =
               Tlink ty2
           | _ ->
               let dup =
-                s.for_saving || more.level = generic_level || static_row row in
+                s.for_saving || more.level = generic_level || static_row row ||
+                match more.desc with Tconstr _ -> true | _ -> false in
               (* Various cases for the row variable *)
               let more' =
-                match more.desc with Tsubst ty -> ty
-                | _ ->
+                match more.desc with
+                  Tsubst ty -> ty
+                | Tconstr _ -> typexp s more
+                | Tunivar | Tvar ->
                     save_desc more more.desc;
                     if s.for_saving then newpersty more.desc else
                     if dup && more.desc <> Tunivar then newgenvar () else more
+                | _ -> assert false
               in
               (* Register new type first for recursion *)
               more.desc <- Tsubst(newgenty(Ttuple[more';ty']));
@@ -201,6 +205,7 @@ let rec class_type s =
 let class_declaration s decl =
   let decl =
     { cty_params = List.map (typexp s) decl.cty_params;
+      cty_variance = decl.cty_variance;
       cty_type = class_type s decl.cty_type;
       cty_path = type_path s decl.cty_path;
       cty_new =
@@ -216,6 +221,7 @@ let class_declaration s decl =
 let cltype_declaration s decl =
   let decl =
     { clty_params = List.map (typexp s) decl.clty_params;
+      clty_variance = decl.clty_variance;
       clty_type = class_type s decl.clty_type;
       clty_path = type_path s decl.clty_path }
   in
